@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: stdctl.c,v 1.3 2003/08/19 20:17:07 f1rmb Exp $
+ * $Id: stdctl.c,v 1.4 2004/04/26 21:31:43 f1rmb Exp $
  *
  * stdctl-specific stuff
  *
@@ -35,6 +35,7 @@
 #include "main.h"
 #include "keys.h"
 #include "actions.h"
+#include "stdctl.h"
 
 #ifdef DEBUG
 #define STDCTL_VERBOSE 1
@@ -49,7 +50,7 @@ typedef struct {
 
 static _stdctl_t stdctl;
 
-void *xine_stdctl_loop(void *dummy) {
+static void *xine_stdctl_loop(void *dummy) {
   char              c[255];
   int               len;
   int               k;
@@ -99,7 +100,7 @@ void *xine_stdctl_loop(void *dummy) {
   pthread_exit(NULL);
 }
 
-void exit_stdctl(void) {
+static void exit_stdctl(void) {
   pthread_cancel(stdctl.thread);
   pthread_join(stdctl.thread, NULL);
 }
