@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: actions.c,v 1.254 2004/12/16 16:14:07 f1rmb Exp $
+ * $Id: actions.c,v 1.261 2005/07/24 02:40:36 fsck-p Exp $
  *
  * implementation of all the various actions for the gui (play, stop, open, pause...)
  *
@@ -40,6 +40,7 @@
 #include <pthread.h>
 
 #include "common.h"
+#include "oxine/oxine.h"
 
 extern gGui_t          *gGui;
 extern _panel_t        *panel;
@@ -64,7 +65,8 @@ void reparent_all_windows(void) {
     { event_sender_is_visible,  event_sender_reparent },
     { stream_infos_is_visible,  stream_infos_reparent },
     { tvset_is_visible,         tvset_reparent },
-    { pplugin_is_visible,       pplugin_reparent },
+    { vpplugin_is_visible,      vpplugin_reparent },
+    { applugin_is_visible,      applugin_reparent },
     { help_is_visible,          help_reparent },
     { NULL,                     NULL}
   };
@@ -76,6 +78,13 @@ void reparent_all_windows(void) {
 
 }
 
+void wait_for_window_visible(Display *display, Window window) {
+  int t = 0;
+
+  while((!xitk_is_window_visible(display, window)) && (++t < 3))
+    xine_usec_sleep(5000);
+}
+
 void raise_window(Window window, int visible, int running) {
   if(window) {
     if(visible && running) {
@@ -87,6 +96,7 @@ void raise_window(Window window, int visible, int running) {
 
 void toggle_window(Window window, xitk_widget_list_t *widget_list, int *visible, int running) {
   if(window && (*visible) && running) {
+
     XLockDisplay(gGui->display);
     if(gGui->use_root_window) {
       if(xitk_is_window_visible(gGui->display, window))
@@ -99,21 +109,27 @@ void toggle_window(Window window, xitk_widget_list_t *widget_list, int *visible,
     }
     else {
       *visible = 0;
+      XUnlockDisplay(gGui->display);
       xitk_hide_widgets(widget_list);
+      XLockDisplay(gGui->display);
       XUnmapWindow(gGui->display, window);
     }
     XUnlockDisplay(gGui->display);
+
   }
   else {
     if(running) {
       *visible = 1;
       xitk_show_widgets(widget_list);
+
       XLockDisplay(gGui->display);
       XRaiseWindow(gGui->display, window);
       XMapWindow(gGui->display, window);
       if(!gGui->use_root_window && gGui->video_display == gGui->display)
         XSetTransientForHint (gGui->display, window, gGui->video_window);
       XUnlockDisplay(gGui->display);
+
+      wait_for_window_visible(gGui->display, window);
       layer_above_video(window);
     }
   }
@@ -156,10 +172,8 @@ int gui_xine_get_pos_length(xine_stream_t *stream, int *pos, int *time, int *len
  *
  */
 void try_to_set_input_focus(Window window) {
-  int t = 0;
 
-  while((!xitk_is_window_visible(gGui->display, window)) && (++t < 3))
-    xine_usec_sleep(5000);
+  wait_for_window_visible(gGui->display, window);
   
   if(xitk_is_window_visible(gGui->display, window)) {
     XLockDisplay (gGui->display);
@@ -223,7 +237,7 @@ static int _gui_xine_play(xine_stream_t *stream,
       gGui->visual_anim.running = 0;
 
   } else if (!has_video && (gGui->visual_anim.enabled == 1) && 
-	     (gGui->visual_anim.running == 0) && gGui->visual_anim.post_output) {
+	     (gGui->visual_anim.running == 0) && gGui->visual_anim.post_output_element.post) {
 
     if(post_rewire_audio_post_to_stream(stream))
       gGui->visual_anim.running = 1;
@@ -557,7 +571,9 @@ int gui_open_and_play_alternates(mediamark_t *mmk, const char *sub) {
  */
 void gui_exit (xitk_widget_t *w, void *data) {
 
-  if(xine_get_status(gGui->stream) != XINE_STATUS_STOP) {
+  oxine_exit();
+
+  if(xine_get_status(gGui->stream) == XINE_STATUS_PLAY) {
     gGui->ignore_next = 1;
 
     if(gGui->visual_anim.running) {
@@ -565,7 +581,7 @@ void gui_exit (xitk_widget_t *w, void *data) {
 
       xine_stop(gGui->visual_anim.stream);
 
-      while(xine_get_status(gGui->visual_anim.stream) != XINE_STATUS_STOP)
+      while(xine_get_status(gGui->visual_anim.stream) == XINE_STATUS_PLAY)
       	xine_usec_sleep(50000);
       
       audio_source = xine_get_audio_source(gGui->stream);
@@ -589,7 +605,8 @@ void gui_exit (xitk_widget_t *w, void *data) {
   event_sender_end();
   stream_infos_end();
   tvset_end();
-  pplugin_end();
+  vpplugin_end();
+  applugin_end();
   help_end();
   download_skin_end();
   
@@ -636,8 +653,8 @@ void gui_exit (xitk_widget_t *w, void *data) {
    */
   gGui->running = 0;
 
-  if(gGui->visual_anim.post_output)
-    xine_post_dispose(gGui->xine, gGui->visual_anim.post_output);
+  if(gGui->visual_anim.post_output_element.post)
+    xine_post_dispose(gGui->xine, gGui->visual_anim.post_output_element.post);
 
   xine_dispose(gGui->stream);
   /* xine_dispose(gGui->visual_anim.stream); */
@@ -905,7 +922,8 @@ static void set_fullscreen_mode(int fullscreen_mode) {
   int event_sender = event_sender_is_visible();
   int stream_infos = stream_infos_is_visible();
   int tvset        = tvset_is_visible();
-  int pplugin      = pplugin_is_visible();
+  int vpplugin     = vpplugin_is_visible();
+  int applugin     = applugin_is_visible();
   int help         = help_is_visible();
 
   if((!(video_window_is_visible())) || gGui->use_root_window)
@@ -932,8 +950,10 @@ static void set_fullscreen_mode(int fullscreen_mode) {
       stream_infos_toggle_visibility(NULL, NULL);
     if(tvset)
       tvset_toggle_visibility(NULL, NULL);
-    if(pplugin)
-      pplugin_toggle_visibility(NULL, NULL);
+    if(vpplugin)
+      vpplugin_toggle_visibility(NULL, NULL);
+    if(applugin)
+      applugin_toggle_visibility(NULL, NULL);
     if(help)
       help_toggle_visibility(NULL, NULL);
   }
@@ -965,8 +985,10 @@ static void set_fullscreen_mode(int fullscreen_mode) {
       stream_infos_toggle_visibility(NULL, NULL);
     if(tvset)
       tvset_toggle_visibility(NULL, NULL);
-    if(pplugin)
-      pplugin_toggle_visibility(NULL, NULL);
+    if(vpplugin)
+      vpplugin_toggle_visibility(NULL, NULL);
+    if(applugin)
+      applugin_toggle_visibility(NULL, NULL);
     if(help)
       help_toggle_visibility(NULL, NULL);
   }
@@ -1132,7 +1154,8 @@ static void *_gui_set_current_position(void *data) {
   
   gGui->ignore_next = 1;
   
-  if(gGui->playlist.num && 
+  if(gGui->playlist.num && gGui->playlist.cur >= 0 && gGui->playlist.mmk &&
+     gGui->playlist.mmk[gGui->playlist.cur] &&
      (!strcmp(gGui->playlist.mmk[gGui->playlist.cur]->mrl, gGui->mmk.mrl)))
     update_mmk = 1;
   
@@ -1260,6 +1283,8 @@ static void *_gui_dndcallback(void *data) {
   int   more_than_one = -2;
   char *mrl           = (char *) data;
 
+  pthread_detach(pthread_self());
+
   if(mrl) {
     char  buffer[strlen(mrl) + 10];
     char  buffer2[strlen(mrl) + 10];
@@ -1547,6 +1572,8 @@ void gui_playlist_show(xitk_widget_t *w, void *data) {
 
 void gui_mrlbrowser_show(xitk_widget_t *w, void *data) {
 
+  gGui->nongui_error_msg = NULL;
+
   if(!mrl_browser_is_running()) {
     open_mrlbrowser(NULL, NULL);
   }
@@ -1688,28 +1715,28 @@ void gui_tvset_show(xitk_widget_t *w, void *data) {
 
 void gui_vpp_show(xitk_widget_t *w, void *data) {
   
-  if (pplugin_is_running() && !pplugin_is_visible())
-    pplugin_toggle_visibility(NULL, NULL);
-  else if(!pplugin_is_running())
-    pplugin_panel();
+  if (vpplugin_is_running() && !vpplugin_is_visible())
+    vpplugin_toggle_visibility(NULL, NULL);
+  else if(!vpplugin_is_running())
+    vpplugin_panel();
   else {
     if(gGui->use_root_window)
-      pplugin_toggle_visibility(NULL, NULL);
+      vpplugin_toggle_visibility(NULL, NULL);
     else
-      pplugin_end();
+      vpplugin_end();
   }
 }
 
 void gui_vpp_enable(void) {
 
-  if(pplugin_is_post_selected()) {
-    gGui->post_enable = !gGui->post_enable;
-    osd_display_info(_("Post plugins: %s."), (gGui->post_enable) ? _("enabled") : _("disabled"));
-    pplugin_update_enable_button();
-    if(pplugin_is_visible())
-      pplugin_rewire_from_posts_window();
+  if(vpplugin_is_post_selected()) {
+    gGui->post_video_enable = !gGui->post_video_enable;
+    osd_display_info(_("Video post plugins: %s."), (gGui->post_video_enable) ? _("enabled") : _("disabled"));
+    vpplugin_update_enable_button();
+    if(vpplugin_is_visible())
+      vpplugin_rewire_from_posts_window();
     else
-      pplugin_rewire_posts();
+      vpplugin_rewire_posts();
   }
 }
 
@@ -1831,6 +1858,33 @@ void gui_decrease_audio_volume(void) {
     gui_decrease_amp_level();
 }
 
+void gui_app_show(xitk_widget_t *w, void *data) {
+  
+  if (applugin_is_running() && !applugin_is_visible())
+    applugin_toggle_visibility(NULL, NULL);
+  else if(!applugin_is_running())
+    applugin_panel();
+  else {
+    if(gGui->use_root_window)
+      applugin_toggle_visibility(NULL, NULL);
+    else
+      applugin_end();
+  }
+}
+
+void gui_app_enable(void) {
+
+  if(applugin_is_post_selected()) {
+    gGui->post_audio_enable = !gGui->post_audio_enable;
+    osd_display_info(_("Audio post plugins: %s."), (gGui->post_audio_enable) ? _("enabled") : _("disabled"));
+    applugin_update_enable_button();
+    if(applugin_is_visible())
+      applugin_rewire_from_posts_window();
+    else
+      applugin_rewire_posts();
+  }
+}
+
 void gui_change_zoom(int zoom_dx, int zoom_dy) {
 
   xine_set_param(gGui->stream, XINE_PARAM_VO_ZOOM_X,
@@ -2020,6 +2074,8 @@ static void fileselector_all_callback(filebrowser_t *fb) {
 void gui_file_selector(void) {
   filebrowser_callback_button_t  cbb[3];
 
+  gGui->nongui_error_msg = NULL;
+
   if(load_stream)
     filebrowser_raise_window(load_stream);
   else {
