@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: common.h,v 1.80 2004/12/15 21:10:43 f1rmb Exp $
+ * $Id: common.h,v 1.82 2005/07/16 19:05:30 miguelfreitas Exp $
  *
  */
 
@@ -158,9 +158,12 @@ typedef struct {
 
   tvout_t                  *tvout;
 
-  post_element_t          **post_elements;
-  int                       post_elements_num;
-  int                       post_enable;
+  post_element_t          **post_video_elements;
+  int                       post_video_elements_num;
+  int                       post_video_enable;
+  post_element_t          **post_audio_elements;
+  int                       post_audio_elements_num;
+  int                       post_audio_enable;
 
   char                     *deinterlace_plugin;
   post_element_t          **deinterlace_elements;
@@ -210,9 +213,8 @@ typedef struct {
     int                     num_mrls;
     
     int                     post_plugin_num;
-    xine_post_t            *post_output;
+    post_element_t          post_output_element;    
     int                     post_changed;
-    
   } visual_anim;
   
   struct {
@@ -357,6 +359,8 @@ typedef struct {
 
   int                        no_gui;
 
+  void                     (*nongui_error_msg)(char *text);
+
 } gGui_t;
 
 
