@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: control.c,v 1.100 2004/04/13 19:55:47 f1rmb Exp $
+ * $Id: control.c,v 1.111 2005/04/13 19:45:08 f1rmb Exp $
  *
  *
  */
@@ -94,6 +94,11 @@ void control_show_tips(int enabled, unsigned long timeout) {
   }
 }
 
+void control_update_tips_timeout(unsigned long timeout) {
+  if(control)
+    xitk_set_widgets_tips_timeout(control->widget_list, timeout);
+}
+
 /*
  * Get current parameter 'param' value.
  */
@@ -257,57 +262,109 @@ static void probe_active_controls(void) {
   contr_ena  = test_vo_property(XINE_PARAM_VO_CONTRAST);
 }
 
+/*
+ * Reset video settings.
+ */
+void control_reset(void) {
+  
+  if(hue_ena) {
+    set_current_param(XINE_PARAM_VO_HUE,
+		      (xitk_slider_get_min(control->hue) +
+		       xitk_slider_get_max(control->hue)) / 2);
+    config_update_num("gui.vo_hue", get_current_param(XINE_PARAM_VO_HUE));
+  }
+
+  if(sat_ena) {
+    set_current_param(XINE_PARAM_VO_SATURATION,
+		      (xitk_slider_get_min(control->sat) +
+		       xitk_slider_get_max(control->sat)) / 2);
+    config_update_num("gui.vo_saturation", get_current_param(XINE_PARAM_VO_SATURATION));
+  }
+
+  if(bright_ena) {
+    set_current_param(XINE_PARAM_VO_BRIGHTNESS,
+		      (xitk_slider_get_min(control->bright) +
+		       xitk_slider_get_max(control->bright)) / 2);
+    config_update_num("gui.vo_brightness", get_current_param(XINE_PARAM_VO_BRIGHTNESS));
+  }
+
+  if(contr_ena) {
+    set_current_param(XINE_PARAM_VO_CONTRAST,
+		      (xitk_slider_get_min(control->contr) +
+		       xitk_slider_get_max(control->contr)) / 2);
+    config_update_num("gui.vo_contrast", get_current_param(XINE_PARAM_VO_CONTRAST));
+  }
+
+  update_sliders_video_settings();
+}
+
 void control_config_register(void) {
   
   probe_active_controls();
   
-  if(hue_ena)
-    set_current_param(XINE_PARAM_VO_HUE,
-		    (gGui->video_settings.hue = 
-		     xine_config_register_range(gGui->xine, "gui.vo_hue",
-						(get_current_param(XINE_PARAM_VO_HUE)),
-						0, 65535,
-						_("hue value"),
-						_("Hue value."), 
-						CONFIG_LEVEL_DEB,
-						hue_changes_cb, 
-						CONFIG_NO_DATA)));
+  if(hue_ena) {
+    gGui->video_settings.hue = 
+     xine_config_register_range(gGui->xine, "gui.vo_hue",
+				-1,
+				0, 65535,
+				_("hue value"),
+				_("Hue value."), 
+				CONFIG_LEVEL_DEB,
+				hue_changes_cb, 
+				CONFIG_NO_DATA);
+    if( gGui->video_settings.hue < 0 )
+      gGui->video_settings.hue = get_current_param(XINE_PARAM_VO_HUE);
+    else 
+      set_current_param(XINE_PARAM_VO_HUE,gGui->video_settings.hue);
+  }
   
-  if(bright_ena)
-    set_current_param(XINE_PARAM_VO_BRIGHTNESS, 
-		    (gGui->video_settings.brightness = 
-		     xine_config_register_range(gGui->xine, "gui.vo_brightness",
-						(get_current_param(XINE_PARAM_VO_BRIGHTNESS)),
-						0, 65535,
-						_("brightness value"),
-						_("Brightness value."), 
-						CONFIG_LEVEL_DEB,
-						brightness_changes_cb, 
-						CONFIG_NO_DATA)));
+  if(bright_ena) {
+    gGui->video_settings.brightness = 
+     xine_config_register_range(gGui->xine, "gui.vo_brightness",
+				-1,
+				0, 65535,
+				_("brightness value"),
+				_("Brightness value."), 
+				CONFIG_LEVEL_DEB,
+				brightness_changes_cb, 
+				CONFIG_NO_DATA);
+    if( gGui->video_settings.brightness < 0 )
+      gGui->video_settings.brightness = get_current_param(XINE_PARAM_VO_BRIGHTNESS);
+    else 
+      set_current_param(XINE_PARAM_VO_BRIGHTNESS,gGui->video_settings.brightness);
+  }
 
-  if(sat_ena)
-    set_current_param(XINE_PARAM_VO_SATURATION,
-		    (gGui->video_settings.saturation = 
-		     xine_config_register_range(gGui->xine, "gui.vo_saturation",
-						(get_current_param(XINE_PARAM_VO_SATURATION)),
-						0, 65535,
-						_("saturation value"),
-						_("Saturation value."), 
-						CONFIG_LEVEL_DEB,
-						saturation_changes_cb, 
-						CONFIG_NO_DATA)));
+  if(sat_ena) {
+   gGui->video_settings.saturation = 
+     xine_config_register_range(gGui->xine, "gui.vo_saturation",
+				-1,
+				0, 65535,
+				_("saturation value"),
+				_("Saturation value."), 
+				CONFIG_LEVEL_DEB,
+				saturation_changes_cb, 
+				CONFIG_NO_DATA);
+    if( gGui->video_settings.saturation < 0 )
+      gGui->video_settings.saturation = get_current_param(XINE_PARAM_VO_SATURATION);
+    else 
+      set_current_param(XINE_PARAM_VO_SATURATION,gGui->video_settings.saturation);
+  }
 
-  if(contr_ena)
-    set_current_param(XINE_PARAM_VO_CONTRAST,
-		    (gGui->video_settings.contrast = 
-		     xine_config_register_range(gGui->xine, "gui.vo_contrast",
-						(get_current_param(XINE_PARAM_VO_CONTRAST)),
-						0, 65535,
-						_("contrast value"),
-						_("Contrast value."), 
-						CONFIG_LEVEL_DEB,
-						contrast_changes_cb, 
-						CONFIG_NO_DATA)));
+  if(contr_ena) {
+    gGui->video_settings.contrast = 
+     xine_config_register_range(gGui->xine, "gui.vo_contrast",
+				-1,
+				0, 65535,
+				_("contrast value"),
+				_("Contrast value."), 
+				CONFIG_LEVEL_DEB,
+				contrast_changes_cb, 
+				CONFIG_NO_DATA);
+    if( gGui->video_settings.contrast < 0 )
+      gGui->video_settings.contrast = get_current_param(XINE_PARAM_VO_CONTRAST);
+    else 
+      set_current_param(XINE_PARAM_VO_CONTRAST,gGui->video_settings.contrast);
+  }
 }
 
 /*
@@ -355,6 +412,8 @@ void control_exit(xitk_widget_t *w, void *data) {
 
     free(control);
     control = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
@@ -378,7 +437,7 @@ int control_is_visible(void) {
     if(gGui->use_root_window)
       return xitk_is_window_visible(gGui->display, control->window);
     else
-      return control->visible;
+      return control->visible && xitk_is_window_visible(gGui->display, control->window);
   }
 
   return 0;
@@ -404,10 +463,20 @@ void control_toggle_visibility (xitk_widget_t *w, void *data) {
 /*
  * Handle X events here.
  */
-void control_handle_event(XEvent *event, void *data) {
+static void control_handle_event(XEvent *event, void *data) {
   
   switch(event->type) {
+  case ButtonPress: {
+    XButtonEvent *bevent = (XButtonEvent *) event;
     
+    if(bevent->button == Button3) {
+      int wx, wy;
+      
+      xitk_get_window_position(gGui->display, control->window, &wx, &wy, NULL, NULL);
+      control_menu(control->widget_list, bevent->x + wx, bevent->y + wy);
+    }
+  }
+
   case KeyPress:
     if(xitk_get_key_pressed(event) == XK_Escape)
       control_exit(NULL, NULL);
@@ -422,7 +491,7 @@ void control_handle_event(XEvent *event, void *data) {
 /*
  * Change the current skin.
  */
-void control_change_skins(void) {
+void control_change_skins(int synthetic) {
   ImlibImage   *new_img, *old_img;
   XSizeHints    hint;
 
@@ -462,7 +531,7 @@ void control_change_skins(void) {
     control->bg_image = new_img;
     
     XLockDisplay(gGui->display);
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       XSetTransientForHint(gGui->display, control->window, gGui->video_window);
     
     Imlib_destroy_image(gGui->imlib_data, old_img);
@@ -494,7 +563,7 @@ void control_deinit(void) {
   if(control) {
     if(control_is_visible())
       control_toggle_visibility(NULL, NULL);
-    xitk_unregister_event_handler(&control->widget_key);
+    control_exit(NULL, NULL);
   }
 }
 
@@ -647,7 +716,7 @@ void control_panel(void) {
   xitk_widget_list_set(control->widget_list, WIDGET_LIST_GC, gc);
   
   { /* All of sliders are disabled by default*/
-    int min = 0, max = 65535, cur;
+    int min = 0, max = 65535, step = 565, cur;
 
     lbl.window = XITK_WIDGET_LIST_WINDOW(control->widget_list);
     lbl.gc     = XITK_WIDGET_LIST_GC(control->widget_list);
@@ -658,7 +727,7 @@ void control_panel(void) {
     sl.skin_element_name = "SliderCtlHue";
     sl.min               = min;
     sl.max               = max;
-    sl.step              = 1;
+    sl.step              = step;
     sl.callback          = set_hue;
     sl.userdata          = NULL;
     sl.motion_callback   = set_hue;
@@ -681,7 +750,7 @@ void control_panel(void) {
     sl.skin_element_name = "SliderCtlSat";
     sl.min               = min;
     sl.max               = max;
-    sl.step              = 1;
+    sl.step              = step;
     sl.callback          = set_saturation;
     sl.userdata          = NULL;
     sl.motion_callback   = set_saturation;
@@ -704,7 +773,7 @@ void control_panel(void) {
     sl.skin_element_name = "SliderCtlBright";
     sl.min               = min;
     sl.max               = max;
-    sl.step              = 1;
+    sl.step              = step;
     sl.callback          = set_brightness;
     sl.userdata          = NULL;
     sl.motion_callback   = set_brightness;
@@ -727,7 +796,7 @@ void control_panel(void) {
     sl.skin_element_name = "SliderCtlCont";
     sl.min               = min;
     sl.max               = max;
-    sl.step              = 1;
+    sl.step              = step;
     sl.callback          = set_contrast;
     sl.userdata          = NULL;
     sl.motion_callback   = set_contrast;
