@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: errors.c,v 1.27 2004/04/15 20:55:36 f1rmb Exp $
+ * $Id: errors.c,v 1.32 2005/02/07 18:16:28 miguelfreitas Exp $
  *
  * xine engine error handling/notification
  *
@@ -59,16 +59,25 @@ static void errors_create_window(char *title, char *message) {
   
   dump_error(gGui->verbosity, message);
 
+  if( gGui->nongui_error_msg ) {
+    gGui->nongui_error_msg( message );
+    return;
+  }
+  
   xw = xitk_window_dialog_two_buttons_with_width(gGui->imlib_data, title, 
 						 _("Done"), _("More..."),
 						 NULL, _errors_display_log, 
 						 NULL, 400, ALIGN_CENTER,
 						 message);
-  if(!gGui->use_root_window) {
+
+  xitk_window_set_parent_window(xw, gGui->video_window);
+
+  if(!gGui->use_root_window && gGui->video_display == gGui->display) {
     XLockDisplay(gGui->display);
     XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
     XUnlockDisplay(gGui->display);
   }
+
   layer_above_video(xitk_window_get_window(xw));
 }
 
@@ -105,20 +114,26 @@ void xine_error(char *message, ...) {
     printf("%s\n", buf);
   }
   else {
-    xitk_window_t *xw;
     char buf2[(strlen(buf) * 2) + 1];
     
     dump_error(gGui->verbosity, buf);
-    
+
     xitk_subst_special_chars(buf, buf2);
-    xw = xitk_window_dialog_error(gGui->imlib_data, buf2);
-    
-    if(!gGui->use_root_window) {
-      XLockDisplay(gGui->display);
-      XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
-      XUnlockDisplay(gGui->display);
+
+    if( gGui->nongui_error_msg ) {
+      gGui->nongui_error_msg( buf2 );
+    } else {
+      xitk_window_t *xw;
+
+      xw = xitk_window_dialog_error(gGui->imlib_data, buf2);
+
+      if(!gGui->use_root_window && gGui->video_display == gGui->display) {
+        XLockDisplay(gGui->display);
+        XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
+        XUnlockDisplay(gGui->display);
+      }
+      layer_above_video(xitk_window_get_window(xw));
     }
-    layer_above_video(xitk_window_get_window(xw));
   }
 
   free(buf);
@@ -160,7 +175,9 @@ void xine_error_with_more(char *message, ...) {
   }
   else {
     char buf2[(strlen(buf) * 2) + 1];
+
     xitk_subst_special_chars(buf, buf2);
+
     errors_create_window(_("Error"), buf2);
   }
   
@@ -202,18 +219,24 @@ void xine_info(char *message, ...) {
     printf("%s\n", buf);
   }
   else {
-    xitk_window_t *xw;
     char           buf2[(strlen(buf) * 2) + 1];
 
     xitk_subst_special_chars(buf, buf2);
-    xw = xitk_window_dialog_info(gGui->imlib_data, buf2);
 
-    if(!gGui->use_root_window) {
-      XLockDisplay(gGui->display);
-      XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
-      XUnlockDisplay(gGui->display);
+    if( gGui->nongui_error_msg ) {
+      gGui->nongui_error_msg( buf2 );
+    } else {
+      xitk_window_t *xw;
+
+      xw = xitk_window_dialog_info(gGui->imlib_data, buf2);
+
+      if(!gGui->use_root_window && gGui->video_display == gGui->display) {
+        XLockDisplay(gGui->display);
+        XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
+        XUnlockDisplay(gGui->display);
+      }
+      layer_above_video(xitk_window_get_window(xw));
     }
-    layer_above_video(xitk_window_get_window(xw));
   }
 
   free(buf);
@@ -280,3 +303,62 @@ void gui_handle_xine_error(xine_stream_t *stream, char *mrl) {
   
   gGui->new_pos = -1;
 }
+
+
+static void _dont_show_too_slow_again(xitk_widget_t *w, void *data, int state) {
+  config_update_num ("gui.dropped_frames_warning", !state);
+}
+
+static void _learn_more_about_too_slow(xitk_widget_t *w, void *data, int state) {
+  /*
+   * fixme: how to properly open the system browser? 
+   * should we just make it configurable? 
+   */
+  xine_info(_("Opening mozilla web browser, this might take a while..."));
+  system ("mozilla http://xinehq.de/index.php/faq#SPEEDUP");
+}
+
+/*
+ * Create the real window.
+ */
+void too_slow_window(void) {
+  xitk_window_t *xw;
+  char *title, *message;
+  int display_warning;
+  int checked = 0;
+    
+  title = _("Warning");
+  message = _("The amount of dropped frame is too high, your system might be slow, not properly optimized or just too loaded.\n\nhttp://xinehq.de/index.php/faq#SPEEDUP");
+  
+  dump_error(gGui->verbosity, message);
+
+  display_warning = xine_config_register_bool (gGui->xine, "gui.dropped_frames_warning", 
+		     1,
+		     _("Warn user when too much frames are dropped."),
+		     CONFIG_NO_HELP,
+		     CONFIG_LEVEL_ADV,
+		     CONFIG_NO_CB,
+		     CONFIG_NO_DATA);
+
+  if( !display_warning )
+    return;
+
+  if( gGui->nongui_error_msg || gGui->stdctl_enable )
+    return;
+      
+  _dont_show_too_slow_again(NULL, NULL, checked);
+    
+  xw = xitk_window_dialog_checkbox_two_buttons_with_width(gGui->imlib_data, title, 
+						 _("Done"), _("Learn More..."),
+						 NULL, _learn_more_about_too_slow, 
+                                                 _("Disable this warning."),
+                                                 checked, _dont_show_too_slow_again,
+						 NULL, 500, ALIGN_CENTER,
+						 message);
+  if(!gGui->use_root_window && gGui->video_display == gGui->display) {
+    XLockDisplay(gGui->display);
+    XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
+    XUnlockDisplay(gGui->display);
+  }
+  layer_above_video(xitk_window_get_window(xw));
+}
