@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: event_sender.c,v 1.23 2004/04/13 19:55:47 f1rmb Exp $
+ * $Id: event_sender.c,v 1.30 2005/05/06 23:32:33 fsck-p Exp $
  *
  *
  */
@@ -129,6 +129,21 @@ static void event_sender_store_new_position(int x, int y, int w, int h) {
   
 }
 
+void event_sender_show_tips(int enabled, unsigned long timeout) {
+  
+  if(eventer) {
+    if(enabled)
+      xitk_set_widgets_tips_timeout(eventer->widget_list, timeout);
+    else
+      xitk_disable_widgets_tips(eventer->widget_list);
+  }
+}
+
+void event_sender_update_tips_timeout(unsigned long timeout) {
+  if(eventer)
+    xitk_set_widgets_tips_timeout(eventer->widget_list, timeout);
+}
+
 /* Send given event to xine engine */
 void event_sender_send(int event) {
   xine_event_t   xine_event;
@@ -197,6 +212,8 @@ static void event_sender_angle(xitk_widget_t *w, void *data) {
   event_sender_send(events[event]);
 }
 
+static void event_sender_exit(xitk_widget_t *, void *);
+
 static void event_sender_handle_event(XEvent *event, void *data) {
   
   switch(event->type) {
@@ -270,6 +287,13 @@ static void event_sender_handle_event(XEvent *event, void *data) {
 	event_sender_num(NULL, (void *)10);
 	break;
 
+      case XK_Escape:
+	event_sender_exit(NULL, NULL);
+	break;
+
+      default:
+	gui_handle_event(event, data);
+	break;
       }
     }
     break;
@@ -291,7 +315,7 @@ void event_sender_update_menu_buttons(void) {
   }
 }
 
-void event_sender_exit(xitk_widget_t *w, void *data) {
+static void event_sender_exit(xitk_widget_t *w, void *data) {
 
   if(eventer) {
     window_info_t wi;
@@ -321,6 +345,8 @@ void event_sender_exit(xitk_widget_t *w, void *data) {
     
     free(eventer);
     eventer = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
@@ -330,7 +356,7 @@ int event_sender_is_visible(void) {
     if(gGui->use_root_window)
       return xitk_is_window_visible(gGui->display, xitk_window_get_window(eventer->xwin));
     else
-      return eventer->visible;
+      return eventer->visible && xitk_is_window_visible(gGui->display, xitk_window_get_window(eventer->xwin));
   }  
 
   return 0;
@@ -368,7 +394,7 @@ void event_sender_move(int x, int y) {
   }
 }
 
-static void event_sender_end(xitk_widget_t *w, void *data) {
+void event_sender_end(void) {
   event_sender_exit(NULL, NULL);
 }
 
@@ -523,7 +549,7 @@ void event_sender_panel(void) {
     char number[2];
     
     memset(&number, 0, sizeof(number));
-    sprintf(number, "%d", i);
+    snprintf(number, sizeof(number), "%d", i);
 
     lb.button_type       = CLICK_BUTTON;
     lb.label             = number;
@@ -560,7 +586,7 @@ void event_sender_panel(void) {
     i = 10;
     
     memset(&number, 0, sizeof(number));
-    sprintf(number, "+ %d", i);
+    snprintf(number, sizeof(number), "+ %d", i);
     
     lb.button_type       = CLICK_BUTTON;
     lb.label             = number;
@@ -666,7 +692,7 @@ void event_sender_panel(void) {
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Close");
   lb.align             = ALIGN_CENTER;
-  lb.callback          = event_sender_end; 
+  lb.callback          = event_sender_exit; 
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
@@ -675,6 +701,7 @@ void event_sender_panel(void) {
 				       &lb, x, y, 70, 23,
 				       "Black", "Black", "White", eventerfontname)));
   xitk_enable_and_show_widget(w);
+  event_sender_show_tips(panel_get_tips_enable(), panel_get_tips_timeout());
 
   eventer->widget_key = xitk_register_event_handler("eventer", 
 						    (xitk_window_get_window(eventer->xwin)),
