@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: help.h,v 1.4 2004/04/13 19:55:47 f1rmb Exp $
+ * $Id: help.h,v 1.5 2004/06/13 11:44:13 f1rmb Exp $
  *
  */
 
@@ -25,7 +25,7 @@
 #define HELP_H
 
 void help_panel(void);
-void help_exit(xitk_widget_t *, void *);
+void help_end(void);
 int help_is_visible(void);
 int help_is_running(void);
 void help_toggle_visibility(xitk_widget_t *, void *);
