@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: lang.h,v 1.7 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: lang.h,v 1.9 2004/04/13 19:55:47 f1rmb Exp $
  *
  */
 
@@ -33,13 +33,15 @@ typedef enum {
   PORTUGUESE,
   SPANISH,
   POLISH,
-  UKRAINIAN
+  UKRAINIAN,
+  CZECH
 } lang_code_t;
 
 typedef struct {
   char         *lang;
   lang_code_t   code;
   char         *ext;
+  char         *doc_encoding;
 } langs_t;
 
 const langs_t *get_lang(void);
