@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mediamark.c,v 1.97 2004/12/16 07:14:26 f1rmb Exp $
+ * $Id: mediamark.c,v 1.101 2005/07/24 02:40:37 fsck-p Exp $
  *
  */
 
@@ -2552,7 +2552,7 @@ void mediamark_insert_entry(int index, const char *mrl, const char *ident,
   if(mrl && (!sub) && gGui->subtitle_autoload) {
 
     if(mrl_look_like_file((char *) mrl)) {
-      char        *know_subs = "sub,srt,asc,smi,ssa";
+      char        *know_subs = "sub,srt,asc,smi,ssa,txt";
       char        *vsubs;
       char        *_mrl, *ending, *ext;
       struct stat  pstat;
@@ -2636,7 +2636,7 @@ int mediamark_all_played(void) {
 int mediamark_get_shuffle_next(void) {
   int  next = 0;
   
-  if(gGui->playlist.num >= 3) {
+  if(gGui->playlist.num >= 3 && gGui->playlist.cur >= 0) {
     int    remain = gGui->playlist.num;
     int    entries[remain];
     float  num = (float) gGui->playlist.num;
@@ -2695,7 +2695,8 @@ void mediamark_replace_entry(mediamark_t **mmk,
 
 mediamark_t *mediamark_get_current_mmk(void) {
 
-  if(gGui->playlist.mmk && gGui->playlist.num)
+  if(gGui->playlist.num && gGui->playlist.cur >= 0 && gGui->playlist.mmk &&
+     gGui->playlist.mmk[gGui->playlist.cur])
     return gGui->playlist.mmk[gGui->playlist.cur];
 
   return (mediamark_t *) NULL;
@@ -2703,7 +2704,8 @@ mediamark_t *mediamark_get_current_mmk(void) {
 
 mediamark_t *mediamark_get_mmk_by_index(int index) {
 
-  if(gGui->playlist.mmk && index < gGui->playlist.num)
+  if(index < gGui->playlist.num && index >= 0 && gGui->playlist.mmk &&
+     gGui->playlist.mmk[index])
     return gGui->playlist.mmk[index];
 
   return (mediamark_t *) NULL;
@@ -2711,7 +2713,9 @@ mediamark_t *mediamark_get_mmk_by_index(int index) {
 
 const char *mediamark_get_current_mrl(void) {
 
-  if(gGui->playlist.mmk && gGui->playlist.num && (gGui->playlist.cur < gGui->playlist.num))
+  if(gGui->playlist.num && gGui->playlist.cur >= 0 && gGui->playlist.mmk &&
+     gGui->playlist.mmk[gGui->playlist.cur] &&
+     gGui->playlist.cur < gGui->playlist.num)
     return gGui->playlist.mmk[gGui->playlist.cur]->mrl;
 
   return NULL;
@@ -2719,7 +2723,8 @@ const char *mediamark_get_current_mrl(void) {
 
 const char *mediamark_get_current_ident(void) {
 
-  if(gGui->playlist.mmk && gGui->playlist.num)
+  if(gGui->playlist.num && gGui->playlist.cur >= 0 && gGui->playlist.mmk &&
+     gGui->playlist.mmk[gGui->playlist.cur])
     return gGui->playlist.mmk[gGui->playlist.cur]->ident;
 
   return NULL;
@@ -2727,7 +2732,8 @@ const char *mediamark_get_current_ident(void) {
 
 const char *mediamark_get_current_sub(void) {
 
-  if(gGui->playlist.mmk && gGui->playlist.num)
+  if(gGui->playlist.num && gGui->playlist.cur >= 0 && gGui->playlist.mmk &&
+     gGui->playlist.mmk[gGui->playlist.cur])
     return gGui->playlist.mmk[gGui->playlist.cur]->sub;
 
   return NULL;
@@ -2735,10 +2741,10 @@ const char *mediamark_get_current_sub(void) {
 
 void mediamark_free_entry(int offset) {
 
-  if(gGui->playlist.num && (offset < gGui->playlist.num)) {
+  if(offset < gGui->playlist.num && offset >= 0 && gGui->playlist.mmk &&
+     gGui->playlist.mmk[offset])
     if(mediamark_free_mmk(&gGui->playlist.mmk[offset]))
       gGui->playlist.num--;
-  }
 }
 
 int mediamark_concat_mediamarks(const char *_filename) {
@@ -3114,18 +3120,19 @@ void mmkeditor_set_mmk(mediamark_t **mmk) {
 }
 
 static void mmkeditor_apply(xitk_widget_t *w, void *data) {
-  const char *ident, *mrl, *sub;
+  const char *sub;
+  char       *ident, *mrl;
   int         start, end, av_offset, spu_offset;
 
   if(mmkeditor->mmk) {
     
     mrl = atoa(xitk_inputtext_get_text(mmkeditor->mrl));
     if(mrl && (!strlen(mrl)))
-      xine_strdupa((char *) mrl, (*mmkeditor->mmk)->mrl);
+      xine_strdupa(mrl, (*mmkeditor->mmk)->mrl);
     
     ident = atoa(xitk_inputtext_get_text(mmkeditor->ident));
     if(ident && (!strlen(ident)))
-      xine_strdupa((char *) ident, mrl);
+      xine_strdupa(ident, mrl);
 
     sub = xitk_inputtext_get_text(mmkeditor->sub);
     if(sub && (!strlen(sub)))
