@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: menus.c,v 1.43 2004/04/13 19:55:48 f1rmb Exp $
+ * $Id: menus.c,v 1.62 2005/07/20 19:08:00 miguelfreitas Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -38,6 +38,7 @@ extern _panel_t                *panel;
 #define PLAYB_PREV              5
 #define PLAYB_SPEEDM            6
 #define PLAYB_SPEEDL            7
+#define PLAYB_ADDMMK            8
 
 #define PLAYL_LOAD              10
 #define PLAYL_SAVE              11
@@ -52,6 +53,8 @@ extern _panel_t                *panel;
 #define AUDIO_MUTE              20
 #define AUDIO_INCRE_VOL         21
 #define AUDIO_DECRE_VOL         22
+#define AUDIO_PPROCESS          23
+#define AUDIO_PPROCESS_ENABLE   24
 
 #define VIDEO_FULLSCR           30
 #define VIDEO_2X                31
@@ -60,6 +63,7 @@ extern _panel_t                *panel;
 #define VIDEO_INTERLEAVE        34
 #define VIDEO_PPROCESS          35
 #define VIDEO_PPROCESS_ENABLE   36
+#define VIDEO_TOGGLE            37
 
 #define SETS_SETUP              40
 #define SETS_KEYMAP             41
@@ -75,9 +79,18 @@ extern _panel_t                *panel;
 
 static char *menu_get_shortcut(char *action) {
   char *shortcut = kbindings_get_shortcut(gGui->kbindings, action);
+#ifdef DEBUG
+  if(!shortcut) {
+    fprintf(stderr, "Action '%s' is invalid\n", action);
+    abort();
+  }
+#endif
   return (shortcut ? strdup(shortcut) : NULL);
 }
 
+static void menu_control_reset(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  control_reset();
+}
 static void menu_open_mrlbrowser(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
   open_mrlbrowser_from_playlist(w, data);
 }
@@ -160,6 +173,10 @@ static void menu_playback_ctrl(xitk_widget_t *w, xitk_menu_entry_t *me, void *da
     gui_execute_action_id(ACTID_SPEED_SLOW);
     break;
 
+  case PLAYB_ADDMMK:
+    gui_execute_action_id(ACTID_ADDMEDIAMARK);
+    break;
+
   default:
     printf("%s(): unknown control %d\n", __XINE_FUNCTION__, ctrl);
     break;
@@ -226,18 +243,16 @@ static void menu_playlist_from(xitk_widget_t *w, xitk_menu_entry_t *me, void *da
     /* Flush playlist in newbie mode */
     if(gGui->smart_mode) {
       mediamark_free_mediamarks();
-      gGui->playlist.cur = 0;
       playlist_update_playlist();
     }
     
-    if(!gGui->playlist.num)
-      gGui->playlist.cur = 0;
-    
     for (j = 0; j < num_mrls; j++)
-      mediamark_add_entry(autoplay_mrls[j], autoplay_mrls[j], NULL, 0, -1, 0, 0);
+      mediamark_append_entry(autoplay_mrls[j], autoplay_mrls[j], NULL, 0, -1, 0, 0);
+    
+    gGui->playlist.cur = gGui->playlist.num ? 0 : -1;
     
     if(gGui->playlist.cur == 0)
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
     
     /* 
      * If we're in newbie mode, start playback immediately
@@ -301,6 +316,14 @@ static void menu_audio_ctrl(xitk_widget_t *w, xitk_menu_entry_t *me, void *data)
     }
     break;
 
+  case AUDIO_PPROCESS:
+    gui_execute_action_id(ACTID_APP);
+    break;
+
+  case AUDIO_PPROCESS_ENABLE:
+    gui_execute_action_id(ACTID_APP_ENABLE);
+    break;
+
   default:
     printf("%s(): unknown control %d\n", __XINE_FUNCTION__, ctrl);
     break;
@@ -360,6 +383,10 @@ static void menu_video_ctrl(xitk_widget_t *w, xitk_menu_entry_t *me, void *data)
     gui_execute_action_id(ACTID_VPP_ENABLE);
     break;
 
+  case VIDEO_TOGGLE:
+    gui_execute_action_id(ACTID_TOGGLE_WINOUT_VISIBLITY);
+    break;
+    
   default:
     printf("%s(): unknown control %d\n", __XINE_FUNCTION__, ctrl);
     break;
@@ -411,8 +438,6 @@ void video_window_menu(xitk_widget_list_t *wl) {
   int                  x, y;
   xitk_menu_widget_t   menu;
   char                 buffer[2048];
-  char                *sh[255];
-  int                  shc = 0;
   xitk_widget_t       *w;
 #ifdef HAVE_XINERAMA
   int                  fullscr_mode = (FULLSCR_MODE | FULLSCR_XI_MODE);
@@ -424,10 +449,18 @@ void video_window_menu(xitk_widget_list_t *wl) {
       NULL,
       "<title>",      
       NULL, NULL                                                                             },
-    { _("GUI visiblity"),
-      (sh[shc++] = menu_get_shortcut("ToggleVisiblity")),
+    { _("Show controls"),
+      menu_get_shortcut("ToggleVisibility"),
       panel_is_visible() ? "<checked>" : "<check>",  
       menu_panel_visibility, NULL                                                            },
+    { _("Show video window"),
+      menu_get_shortcut("ToggleWindowVisibility"),
+      video_window_is_visible() ? "<checked>" : "<check>",  
+      menu_video_ctrl, (void *) VIDEO_TOGGLE                                                 },
+    { _("Fullscreen"),
+      menu_get_shortcut("ToggleFullscreen"),
+      (video_window_get_fullscreen_mode() & fullscr_mode) ? "<checked>" : "<check>",
+      menu_video_ctrl, (void *) VIDEO_FULLSCR                                                },
     { "SEP",  
       NULL,
       "<separator>",  
@@ -437,7 +470,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
       "<branch>",   
       NULL, NULL                                                                             },
     { _("Open/File..."),
-      (sh[shc++] = menu_get_shortcut("FileSelector")),
+      menu_get_shortcut("FileSelector"),
       NULL,
       menu_file_selector,        NULL                                                        },
     { _("Open/Playlist..."),
@@ -445,7 +478,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
       NULL,
       menu_playlist_ctrl, (void *) PLAYL_LOAD                                                },
     { _("Open/Location..."),
-      (sh[shc++] = menu_get_shortcut("MrlBrowser")),
+      menu_get_shortcut("MrlBrowser"),
       NULL,
       menu_mrl_browser, NULL                                                                 },
     { _("Playback"),
@@ -453,15 +486,15 @@ void video_window_menu(xitk_widget_list_t *wl) {
       "<Branch>",
       NULL, NULL                                                                             },
     { _("Playback/Play"),
-      (sh[shc++] = menu_get_shortcut("Play")),
+      menu_get_shortcut("Play"),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_PLAY                                                },
     { _("Playback/Stop"),
-      (sh[shc++] = menu_get_shortcut("Stop")),
+      menu_get_shortcut("Stop"),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_STOP                                                },
     { _("Playback/Pause"),
-      (sh[shc++] = menu_get_shortcut("Pause")),
+      menu_get_shortcut("Pause"),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_PAUSE                                               },
     { _("Playback/SEP"),
@@ -469,11 +502,11 @@ void video_window_menu(xitk_widget_list_t *wl) {
       "<separator>",  
       NULL,  NULL                                                                            },
     { _("Playback/Next MRL"),
-      (sh[shc++] = menu_get_shortcut("NextMrl")),
+      menu_get_shortcut("NextMrl"),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_NEXT                                                },
     { _("Playback/Previous MRL"),
-      (sh[shc++] = menu_get_shortcut("PreviousMrl")),
+      menu_get_shortcut("PriorMrl"),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_PREV                                                },
     { _("Playback/SEP"),
@@ -481,11 +514,11 @@ void video_window_menu(xitk_widget_list_t *wl) {
       "<separator>",  
       NULL,  NULL                                                                            },
     { _("Playback/Increase Speed"),
-      (sh[shc++] = menu_get_shortcut("SpeedFaster")),
+      menu_get_shortcut("SpeedFaster"),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_SPEEDM                                              },
     { _("Playback/Decrease Speed"),
-      (sh[shc++] = menu_get_shortcut("SpeedSlower")),
+      menu_get_shortcut("SpeedSlower"),
       NULL,
       menu_playback_ctrl, (void *) PLAYB_SPEEDL                                              },
     { _("Playlist"),
@@ -501,7 +534,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
       NULL,
       menu_playlist_ctrl, (void *) PLAYL_LOAD                                                },
     { _("Playlist/Editor..."),
-      (sh[shc++] = menu_get_shortcut("PlaylistEditor")),
+      menu_get_shortcut("PlaylistEditor"),
       NULL,
       menu_playlist_ctrl, (void *) PLAYL_EDIT                                                },
     { _("Playlist/SEP"),  
@@ -532,12 +565,8 @@ void video_window_menu(xitk_widget_list_t *wl) {
       NULL,
       (gGui->playlist.loop == PLAYLIST_LOOP_SHUF_PLUS) ? "<checked>" : "<check>",
       menu_playlist_ctrl, (void *) PLAYL_SHUF_PLUS                                           },
-    { _("Playlist/SEP"),  
-      NULL,
-      "<separator>",
-      NULL, NULL                                                                             },
-    { _("Playlist/Continue"),
-      (sh[shc++] = menu_get_shortcut("PlaylistStop")),
+    { _("Playlist/Continue Playback"),
+      menu_get_shortcut("PlaylistStop"),
       (gGui->playlist.control & PLAYLIST_CONTROL_STOP) ? "<check>" : "<checked>",
       menu_playlist_ctrl, (void *) PLAYL_CTRL_STOP                                           },
     { "SEP",  
@@ -549,7 +578,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
       "<branch>",
       NULL, NULL                                                                             },
     { _("Menus/Navigation..."),
-      (sh[shc++] = menu_get_shortcut("EventSenderShow")),
+      menu_get_shortcut("EventSenderShow"),
       NULL,
       menu_event_sender, NULL                                                                },
     { _("Menus/SEP"),  
@@ -565,24 +594,20 @@ void video_window_menu(xitk_widget_list_t *wl) {
       "<branch>",
       NULL, NULL                                                                             },
     { _("Stream/Information..."),
-      (sh[shc++] = menu_get_shortcut("StreamInfosShow")),
+      menu_get_shortcut("StreamInfosShow"),
       NULL,
       menu_stream, (void *) STREAM_WINI                                                      },
     { _("Stream/Information (OSD)"),
-      (sh[shc++] = menu_get_shortcut("OSDStreamInfos")),
+      menu_get_shortcut("OSDStreamInfos"),
       NULL,
       menu_stream, (void *) STREAM_OSDI                                                      },
-    { _("Fullscreen\\/Window"),
-      (sh[shc++] = menu_get_shortcut("ToggleFullscreen")),
-      (video_window_get_fullscreen_mode() & fullscr_mode) ? "<checked>" : "<check>",
-      menu_video_ctrl, (void *) VIDEO_FULLSCR                                                },
     { _("Video"),
       NULL,
       "<branch>",
       NULL, NULL                                                                             },
     { _("Video/Deinterlace"),
-      (sh[shc++] = menu_get_shortcut("ToggleInterleave")),
-      (xine_get_param(gGui->stream, XINE_PARAM_VO_DEINTERLACE)) ? "<checked>" : "<check>",
+      menu_get_shortcut("ToggleInterleave"),
+      (gGui->deinterlace_enable) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_INTERLEAVE                                             },
     { _("Video/SEP"),
       NULL,
@@ -613,15 +638,15 @@ void video_window_menu(xitk_widget_list_t *wl) {
       (aspect == XINE_VO_ASPECT_DVB) ? "<checked>" : "<check>",
       menu_aspect, (void *) XINE_VO_ASPECT_DVB                                               },
     { _("Video/200%"),
-      (sh[shc++] = menu_get_shortcut("Window200")),
+      menu_get_shortcut("Window200"),
       (video_window_get_mag() == 2.0) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_2X                                                     },
     { _("Video/100%"),
-      (sh[shc++] = menu_get_shortcut("Window100")),
+      menu_get_shortcut("Window100"),
       (video_window_get_mag() == 1.0) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_1X                                                     },
     { _("Video/50%"),
-      (sh[shc++] = menu_get_shortcut("Window50")),
+      menu_get_shortcut("Window50"),
       (video_window_get_mag() ==  .5) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO__5X                                                    },
     { _("Video/SEP"),
@@ -633,12 +658,12 @@ void video_window_menu(xitk_widget_list_t *wl) {
       "<branch>",
       NULL, NULL                                                                             },
     { _("Video/Postprocess/Chain Reaction..."),
-      (sh[shc++] = menu_get_shortcut("VPProcessShow")),
+      menu_get_shortcut("VPProcessShow"),
       NULL,
       menu_video_ctrl, (void *) VIDEO_PPROCESS                                               },
     { _("Video/Postprocess/Enable Postprocessing"),
-      (sh[shc++] = menu_get_shortcut("VPProcessEnable")),
-      gGui->post_enable ? "<checked>" : "<check>",
+      menu_get_shortcut("VPProcessEnable"),
+      gGui->post_video_enable ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_PPROCESS_ENABLE                                        },
     { _("Audio"),
       NULL,
@@ -649,7 +674,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
       "<branch>",
       NULL, NULL                                                                             },
     { _("Audio/Volume/Mute"),
-      (sh[shc++] = menu_get_shortcut("Mute")),
+      menu_get_shortcut("Mute"),
       gGui->mixer.mute ? "<checked>" : "<check>",
       menu_audio_ctrl, (void *) AUDIO_MUTE                                                   },
     { _("Audio/Volume/Increase 10%"),
@@ -668,6 +693,22 @@ void video_window_menu(xitk_widget_list_t *wl) {
       NULL,
       "<branch>",
       NULL, NULL                                                                             },
+    { _("Audio/SEP"),
+      NULL,
+      "<separator>",
+      NULL, NULL                                                                             },
+    { _("Audio/Postprocess"),
+      NULL,
+      "<branch>",
+      NULL, NULL                                                                             },
+    { _("Audio/Postprocess/Chain Reaction..."),
+      NULL /* menu_get_shortcut("APProcessShow") */,
+      NULL,
+      menu_audio_ctrl, (void *) AUDIO_PPROCESS                                               },
+    { _("Audio/Postprocess/Enable Postprocessing"),
+      NULL /* menu_get_shortcut("APProcessEnable") */,
+      gGui->post_audio_enable ? "<checked>" : "<check>",
+      menu_audio_ctrl, (void *) AUDIO_PPROCESS_ENABLE                                        },
     { _("Subtitle"),
       NULL,
       "<branch>",
@@ -685,25 +726,25 @@ void video_window_menu(xitk_widget_list_t *wl) {
       "<branch>",
       NULL, NULL                                                                             },
     { _("Settings/Setup..."),
-      (sh[shc++] = menu_get_shortcut("SetupShow")),
+      menu_get_shortcut("SetupShow"),
       NULL,
       menu_settings, (void *) SETS_SETUP                                                     },
 #ifdef HAVE_CURL
     { _("Settings/Skin Downloader..."),
-      (sh[shc++] = menu_get_shortcut("SkinDownload")),
+      menu_get_shortcut("SkinDownload"),
       NULL,
       menu_settings, (void *) SETS_SKINDL                                                    },
 #endif
     { _("Settings/Keymap Editor..."),
-      (sh[shc++] = menu_get_shortcut("KeyBindingEditor")),
+      menu_get_shortcut("KeyBindingEditor"),
       NULL,
       menu_settings, (void *) SETS_KEYMAP                                                    },
     { _("Settings/Video..."),
-      (sh[shc++] = menu_get_shortcut("ControlShow")),
+      menu_get_shortcut("ControlShow"),
       NULL,
       menu_settings, (void *) SETS_VIDEO                                                     },
     { _("Settings/TV Analog..."),
-      (sh[shc++] = menu_get_shortcut("TVAnalogShow")),
+      menu_get_shortcut("TVAnalogShow"),
       NULL,
       menu_settings, (void *) SETS_TVANALOG                                                  },
     { "SEP",
@@ -711,11 +752,11 @@ void video_window_menu(xitk_widget_list_t *wl) {
       "<separator>",
       NULL, NULL                                                                             },
     { _("Help..."),
-      (sh[shc++] = menu_get_shortcut("HelpShow")),
+      menu_get_shortcut("HelpShow"),
       NULL,
       menu_help, NULL                                                                        },
     { _("Logs..."),
-      (sh[shc++] = menu_get_shortcut("ViewlogShow")),
+      menu_get_shortcut("ViewlogShow"),
       NULL,
       menu_settings, (void *) SETS_LOGS                                                      },
     { "SEP",
@@ -723,7 +764,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
       "<separator>",
       NULL, NULL                                                                             },
     { _("Quit"),
-      (sh[shc++] = menu_get_shortcut("Quit")),
+      menu_get_shortcut("Quit"),
       NULL,
       menu_quit, NULL                                                                        },
     { NULL,
@@ -731,22 +772,35 @@ void video_window_menu(xitk_widget_list_t *wl) {
       NULL,
       NULL, NULL                                                                             }
   };
+  xitk_menu_entry_t *cur_entry;
 
-  sprintf(buffer, _("xine %s"), VERSION);
+  if(gGui->no_gui)
+    return;
+
+  gGui->nongui_error_msg = NULL;
+  
+  snprintf(buffer, sizeof(buffer), _("xine %s"), VERSION);
   menu_entries[0].menu = buffer;
 
   XITK_WIDGET_INIT(&menu, gGui->imlib_data);
-  (void) xitk_get_mouse_coords(gGui->display, 
-			       gGui->video_window, NULL, NULL, &x, &y);
   
+  XLockDisplay(gGui->video_display);
+  (void) xitk_get_mouse_coords(gGui->video_display, 
+			       gGui->video_window, NULL, NULL, &x, &y);
+  XUnlockDisplay(gGui->video_display);
+
   menu.menu_tree         = &menu_entries[0];
   menu.parent_wlist      = wl;
   menu.skin_element_name = NULL;
   
   w = xitk_noskin_menu_create(wl, &menu, x + 1, y + 1);
 
-  while(shc > 0)
-    free(sh[--shc]);
+  cur_entry = &menu_entries[0];
+  while(cur_entry->menu) {
+    if(cur_entry->shortcut)
+      free(cur_entry->shortcut);
+    cur_entry++;
+  }
 
   /* Subtitle loader */
   if(gGui->playlist.num) {
@@ -776,7 +830,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
 
       memset(&buffer, 0, sizeof(buffer));
       memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
-      sprintf(buffer, "%s/%s", location, plug_id);
+      snprintf(buffer, sizeof(buffer), "%s/%s", location, plug_id);
       
       menu_entry.menu      = buffer;
       menu_entry.cb        = menu_playlist_from;
@@ -797,7 +851,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
 
       while(viz_names[i]) {
 	memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
-	sprintf(buffer, "%s/%s", location, viz_names[i]);
+	snprintf(buffer, sizeof(buffer), "%s/%s", location, viz_names[i]);
 	menu_entry.menu      = buffer;
 	menu_entry.type      = IS_CHANNEL_CHECKED(i, gGui->visual_anim.post_plugin_num);
 	menu_entry.cb        = menu_audio_viz;
@@ -846,7 +900,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
 	if(i == 0) {
 	  for(i = 0; i < 15; i++) {
 	    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
-	    sprintf(buffer, "%s/%d", location, i);
+	    snprintf(buffer, sizeof(buffer), "%s/%d", location, i);
 	    menu_entry.menu      = buffer;
 	    menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
 	    menu_entry.cb        = menu_audio_chan;
@@ -859,7 +913,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
       }
       
       memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
-      sprintf(buffer, "%s/%s", location, (get_language_from_iso639_1(langbuf)));
+      snprintf(buffer, sizeof(buffer), "%s/%s", location, (get_language_from_iso639_1(langbuf)));
       menu_entry.menu      = buffer;
       menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
       menu_entry.cb        = menu_audio_chan;
@@ -900,7 +954,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
 	if(i == 0) {
 	  for(i = 0; i < 15; i++) {
 	    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
-	    sprintf(buffer, "%s/%d", location, i);
+	    snprintf(buffer, sizeof(buffer), "%s/%d", location, i);
 	    menu_entry.menu      = buffer;
 	    menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
 	    menu_entry.cb        = menu_spu_chan;
@@ -913,7 +967,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
       }
       
       memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
-      sprintf(buffer, "%s/%s", location, (get_language_from_iso639_1(langbuf)));
+      snprintf(buffer, sizeof(buffer), "%s/%s", location, (get_language_from_iso639_1(langbuf)));
       menu_entry.menu      = buffer;
       menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
       menu_entry.cb        = menu_spu_chan;
@@ -942,7 +996,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
       menu = dvd_menu;
     
     for(i = 0; i < 7; i++) {
-      sprintf(buffer, "%s/%s", location, menu[i]);
+      snprintf(buffer, sizeof(buffer), "%s/%s", location, menu[i]);
       
       memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
       menu_entry.menu      = buffer;
@@ -952,6 +1006,31 @@ void video_window_menu(xitk_widget_list_t *wl) {
     }
   }
 
+  /* Mediamark */
+  if(xine_get_status(gGui->stream) != XINE_STATUS_STOP) {
+    xitk_menu_entry_t   menu_entry;
+    char                buffer[2048];
+    char               *location = _("Playback");
+
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
+    menu_entry.menu      = _("Playback/SEP");
+    menu_entry.type      = "<separator>";
+    xitk_menu_add_entry(w, &menu_entry);
+    
+    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
+    memset(&buffer, 0, sizeof(buffer));
+    
+    snprintf(buffer, sizeof(buffer), "%s/%s", location, _("Add Mediamark"));
+    menu_entry.menu      = buffer;
+    menu_entry.shortcut  = menu_get_shortcut("AddMediamark");
+    menu_entry.cb        = menu_playback_ctrl;
+    menu_entry.user_data = (void *) PLAYB_ADDMMK;
+    xitk_menu_add_entry(w, &menu_entry);
+    
+    if(menu_entry.shortcut)
+      free(menu_entry.shortcut);
+  }
+
   xitk_menu_show_menu(w);
 }
 
@@ -965,7 +1044,7 @@ void audio_lang_menu(xitk_widget_list_t *wl, int x, int y) {
     { NULL,  NULL, NULL,           NULL, NULL }
   };
 
-  sprintf(buffer, "%s", _("Audio"));
+  snprintf(buffer, sizeof(buffer), "%s", _("Audio"));
   menu_entries[0].menu = buffer;
   
   XITK_WIDGET_INIT(&menu, gGui->imlib_data);
@@ -1006,7 +1085,7 @@ void audio_lang_menu(xitk_widget_list_t *wl, int x, int y) {
 	if(i == 0) {
 	  for(i = 0; i < 15; i++) {
 	    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
-	    sprintf(buffer, "%d", i);
+	    snprintf(buffer, sizeof(buffer), "%d", i);
 	    menu_entry.menu      = buffer;
 	    menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
 	    menu_entry.cb        = menu_audio_chan;
@@ -1041,7 +1120,7 @@ void spu_lang_menu(xitk_widget_list_t *wl, int x, int y) {
     { NULL,  NULL, NULL,           NULL, NULL }
   };
 
-  sprintf(buffer, "%s", _("Subtitle"));
+  snprintf(buffer, sizeof(buffer), "%s", _("Subtitle"));
   menu_entries[0].menu = buffer;
   
   XITK_WIDGET_INIT(&menu, gGui->imlib_data);
@@ -1081,7 +1160,7 @@ void spu_lang_menu(xitk_widget_list_t *wl, int x, int y) {
 	if(i == 0) {
 	  for(i = 0; i < 15; i++) {
 	    memset(&menu_entry, 0, sizeof(xitk_menu_entry_t));
-	    sprintf(buffer, "%d", i);
+	    snprintf(buffer, sizeof(buffer), "%d", i);
 	    menu_entry.menu      = buffer;
 	    menu_entry.type      = IS_CHANNEL_CHECKED(channel, i);
 	    menu_entry.cb        = menu_spu_chan;
@@ -1113,7 +1192,8 @@ void playlist_menu(xitk_widget_list_t *wl, int x, int y, int selected) {
   xitk_menu_entry_t    menu_entries_nosel[] = {
     { NULL ,           NULL,          "<title>",     NULL,                         NULL                    },
     { "SEP",           NULL,          "<separator>", NULL,                         NULL                    },
-    { _("Scan"),       NULL,          NULL,          menu_scan_infos,              NULL                    },
+    { _("Scan"),       menu_get_shortcut("ScanPlaylistInfo"),
+                                      NULL,          menu_scan_infos,              NULL                    },
     { _("Add"),        NULL,          NULL,          menu_open_mrlbrowser,         NULL                    },
     { NULL,            NULL,          NULL,          NULL,                         NULL                    }
   };
@@ -1124,7 +1204,8 @@ void playlist_menu(xitk_widget_list_t *wl, int x, int y, int selected) {
     { "SEP",           NULL,          "<separator>", NULL,                         NULL                    },
     { _("Scan"),       NULL,          NULL,          menu_scan_infos_selected,     NULL                    },
     { _("Add"),        NULL,          NULL,          menu_open_mrlbrowser,         NULL                    },
-    { _("Edit"),       NULL,          NULL,          menu_playlist_mmk_editor,     NULL                    },
+    { _("Edit"),       menu_get_shortcut("MediamarkEditor"),
+                                      NULL,          menu_playlist_mmk_editor,     NULL                    },
     { _("Delete"),     NULL,          NULL,          menu_playlist_delete_current, NULL                    },
     { _("Delete All"), NULL,          NULL,          menu_playlist_delete_all,     NULL                    },
     { "SEP",           NULL,          "<separator>", NULL,                         NULL                    },
@@ -1132,11 +1213,12 @@ void playlist_menu(xitk_widget_list_t *wl, int x, int y, int selected) {
     { _("Move Down"),  NULL,          NULL,          menu_playlist_move_updown,    (void *) DIRECTION_DOWN },
     { NULL,            NULL,          NULL,          NULL,                         NULL                    }
   };
+  xitk_menu_entry_t *cur_entry;
 
   
   XITK_WIDGET_INIT(&menu, gGui->imlib_data);
   
-  sprintf(buffer, "%s", _("Playlist"));
+  snprintf(buffer, sizeof(buffer), "%s", _("Playlist"));
   if(selected) {
     menu_entries_sel[0].menu = buffer;
     menu.menu_tree           = &menu_entries_sel[0];
@@ -1151,6 +1233,13 @@ void playlist_menu(xitk_widget_list_t *wl, int x, int y, int selected) {
 
   w = xitk_noskin_menu_create(wl, &menu, x, y);
 
+  cur_entry = &menu_entries_sel[0];
+  while(cur_entry->menu) {
+    if(cur_entry->shortcut)
+      free(cur_entry->shortcut);
+    cur_entry++;
+  }
+  
   if(!selected && gGui->playlist.num) {
     xitk_menu_entry_t   menu_entry;
     
@@ -1162,3 +1251,28 @@ void playlist_menu(xitk_widget_list_t *wl, int x, int y, int selected) {
 
   xitk_menu_show_menu(w);
 }
+
+void control_menu(xitk_widget_list_t *wl, int x, int y) {
+  xitk_menu_widget_t   menu;
+  xitk_widget_t       *w = NULL;
+  char                 buffer[2048];
+  xitk_menu_entry_t    menu_entries[] = {
+    { NULL ,           NULL,          "<title>",     NULL,                         NULL                    },
+    { "SEP",           NULL,          "<separator>", NULL,                         NULL                    },
+    { _("Reset video settings"),   
+                       NULL,          NULL,          menu_control_reset,           NULL                    },
+    { NULL,            NULL,          NULL,          NULL,                         NULL                    }
+  };
+  
+  XITK_WIDGET_INIT(&menu, gGui->imlib_data);
+  
+  snprintf(buffer, sizeof(buffer), "%s", _("Video Control"));
+  menu_entries[0].menu   = buffer;
+  menu.menu_tree         = &menu_entries[0];
+  menu.parent_wlist      = wl;
+  menu.skin_element_name = NULL;
+
+  w = xitk_noskin_menu_create(wl, &menu, x, y);
+  
+  xitk_menu_show_menu(w);
+}
