@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrl_browser.c,v 1.70 2004/04/13 19:55:48 f1rmb Exp $
+ * $Id: mrl_browser.c,v 1.77 2005/04/16 02:32:14 fsck-p Exp $
  *
  * MRL Browser
  *
@@ -46,13 +46,21 @@ void mrl_browser_show_tips(int enabled, unsigned long timeout) {
 
 }
 
+void mrl_browser_update_tips_timeout(unsigned long timeout) {
+  if(mrlb) {
+    if(xitk_get_widget_tips_timeout(mrlb) > 0)
+      xitk_mrlbrowser_set_tips_timeout(mrlb, 1, timeout);
+
+  }
+}
+
 /*
  *
  */
 void set_mrl_browser_transient(void) {
 
   if(mrlb) {
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       xitk_mrlbrowser_set_transient(mrlb, gGui->video_window);
   }
 }
@@ -60,7 +68,7 @@ void set_mrl_browser_transient(void) {
 /*
  *
  */
-void mrl_browser_change_skins(void) {
+void mrl_browser_change_skins(int synthetic) {
 
   if(mrlb) {
     xitk_mrlbrowser_change_skins(mrlb, gGui->skin_config);
@@ -98,6 +106,7 @@ int mrl_browser_is_running(void) {
 void show_mrl_browser(void) {
 
   if(mrlb) {
+    gGui->nongui_error_msg = NULL;
     xitk_mrlbrowser_show(mrlb);
     set_mrl_browser_transient();
     layer_above_video((xitk_mrlbrowser_get_window_id(mrlb)));
@@ -142,6 +151,8 @@ void destroy_mrl_browser(void) {
     }
     xitk_mrlbrowser_destroy(mrlb);
     mrlb = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
@@ -157,13 +168,15 @@ static void mrl_browser_kill(xitk_widget_t *w, void *data) {
       config_update_num ("gui.mrl_browser_y", wi.y);
       WINDOW_INFO_ZERO(&wi);
     }
+
+    try_to_set_input_focus(gGui->video_window);
   }
 
   /* FIXME */
   mrlb = NULL;
 }
 
-xitk_mrlbrowser_filter_t **mrl_browser_get_valid_mrl_ending(void) {
+static xitk_mrlbrowser_filter_t **mrl_browser_get_valid_mrl_ending(void) {
   xitk_mrlbrowser_filter_t **filters = NULL;
   int                        num_endings = 0;
   char                      *mrl_exts, *p, *pp;
@@ -242,7 +255,7 @@ void mrl_browser(xitk_mrl_callback_t add_cb, xitk_mrl_callback_t play_cb,
 
   XITK_WIDGET_INIT(&mb, gGui->imlib_data);
 
-  mb.window_trans                   = (gGui->use_root_window) ? None : gGui->video_window;
+  mb.window_trans                   = (gGui->use_root_window || gGui->video_display != gGui->display) ? None : gGui->video_window;
   mb.layer_above                    = (is_layer_above());
   mb.icon                           = &gGui->icon;
   mb.set_wm_window_normal           = !video_window_is_visible();
@@ -349,11 +362,11 @@ static void mrl_add_noautoplay(xitk_widget_t *w, void *data, xine_mrl_t *mrl) {
 	playlist_toggle_visibility(NULL, NULL);
     }
     
-    mediamark_add_entry((char *)mrl->mrl, (char *)mrl->mrl, NULL, 0, -1, 0, 0);
+    mediamark_append_entry((char *)mrl->mrl, (char *)mrl->mrl, NULL, 0, -1, 0, 0);
     
     if((!num) && ((xine_get_status(gGui->stream) == XINE_STATUS_STOP) || gGui->logo_mode)) {
       gGui->playlist.cur = (gGui->playlist.num - 1);
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      gui_set_current_mmk(mediamark_get_current_mmk());
     }   
     
     playlist_update_playlist();
@@ -398,7 +411,7 @@ static void mrl_play(xitk_widget_t *w, void *data, xine_mrl_t *mrl) {
 
     if(mrl_look_like_playlist(_mrl)) {
       if(mediamark_concat_mediamarks(_mrl)) {
-	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	gui_set_current_mmk(mediamark_get_current_mmk());
 	_mrl = (char *) mediamark_get_current_mrl();
 	playlist_update_playlist();
       }
@@ -417,12 +430,17 @@ static void mrl_play(xitk_widget_t *w, void *data, xine_mrl_t *mrl) {
       return;
     }
     
-    mmk.mrl   = _mrl;
-    mmk.ident = NULL;
-    mmk.sub   = NULL;
-    mmk.start = 0;
-    mmk.end   = -1;
-    gui_set_current_mrl(&mmk);
+    mmk.mrl           = _mrl;
+    mmk.ident         = NULL;
+    mmk.sub           = NULL;
+    mmk.start         = 0;
+    mmk.end           = -1;
+    mmk.av_offset     = 0;
+    mmk.spu_offset    = 0;
+    mmk.got_alternate = 0;
+    mmk.alternates    = NULL;
+    mmk.cur_alt       = NULL;
+    gui_set_current_mmk(&mmk);
   }
 }
 
