@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: osd.h,v 1.11 2004/04/13 19:55:48 f1rmb Exp $
+ * $Id: osd.h,v 1.13 2005/06/05 02:48:31 fsck-p Exp $
  *
  */
 
@@ -26,12 +26,25 @@
 
 #include <stdarg.h>
 
+typedef struct {
+  xine_osd_t   *osd[2];
+  int           visible;
+  int           have_text;
+  int           x, y;
+  int           w, h;
+} osd_object_t;
+
 #define OSD_BAR_PROGRESS 1
 #define OSD_BAR_POS      2
 #define OSD_BAR_POS2     3
 #define OSD_BAR_STEPPER  4
 
 void osd_init(void);
+void osd_hide_sinfo(void);
+void osd_hide_bar(void);
+void osd_hide_status(void);
+void osd_hide_info(void);
+void osd_hide(void);
 void osd_deinit(void);
 void osd_update(void);
 void osd_display_spu_lang(void);
@@ -40,6 +53,11 @@ void osd_stream_infos(void);
 void osd_update_status(void);
 void osd_stream_position(int pos);
 void osd_display_info(char *info, ...);
-  /* see OSD_BAR_* */
+
+/* see OSD_BAR_* */
 void osd_draw_bar(char *title, int min, int max, int val, int type);
+
+int osd_is_visible(void);
+void osd_update_osd(void);
+
 #endif
