@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: panel.c,v 1.217 2004/12/15 21:10:45 f1rmb Exp $
+ * $Id: panel.c,v 1.226 2005/07/24 02:40:37 fsck-p Exp $
  *
  * xine panel related stuff
  *
@@ -111,13 +111,7 @@ static void panel_enable_tips_cb(void *data, xine_cfg_entry_t *cfg) {
 static void panel_timeout_tips_cb(void *data, xine_cfg_entry_t *cfg) {
   panel->tips.timeout = (unsigned long) cfg->num_value;
 
-  xitk_set_widgets_tips_timeout(panel->widget_list, panel->tips.timeout);
-  control_update_tips_timeout(panel->tips.timeout);
-  event_sender_update_tips_timeout(panel->tips.timeout);
-  mmk_editor_update_tips_timeout(panel->tips.timeout);
-  mrl_browser_update_tips_timeout(panel->tips.timeout);
-  playlist_update_tips_timeout(panel->tips.timeout);
-  setup_update_tips_timeout(panel->tips.timeout);
+  panel_show_tips();
 }
 
 /*
@@ -162,6 +156,7 @@ static void panel_exit(xitk_widget_t *w, void *data) {
     XUnmapWindow(gGui->display, gGui->panel_window);
     XUnlockDisplay(gGui->display);
 
+    panel->title_label = 0;
     xitk_destroy_widgets(panel->widget_list);
 
     XLockDisplay(gGui->display);
@@ -306,6 +301,7 @@ void panel_update_runtime_display(void) {
     return;
 
   if(!gui_xine_get_pos_length(gGui->stream, &pos, &seconds, &length)) {
+
     if((gGui->stream_length.pos || gGui->stream_length.time) && gGui->stream_length.length) {
       pos     = gGui->stream_length.pos;
       seconds = gGui->stream_length.time;
@@ -313,6 +309,7 @@ void panel_update_runtime_display(void) {
     }
     else
       return;
+
   }
 
   if(pos || seconds) {
@@ -388,7 +385,8 @@ static void *slider_loop(void *dummy) {
 
 	  pthread_mutex_lock(&gGui->xe_mutex);
 
-	  if(gGui->playlist.num && gGui->mmk.end != -1) {
+	  if(gGui->playlist.num && gGui->playlist.cur >= 0 && gGui->playlist.mmk &&
+	     gGui->playlist.mmk[gGui->playlist.cur] && gGui->mmk.end != -1) {
 	    if(secs >= gGui->playlist.mmk[gGui->playlist.cur]->end) {
 	      gGui->ignore_next = 0;
 	      gui_playlist_start_next();
@@ -431,18 +429,21 @@ static void *slider_loop(void *dummy) {
 	char *ident = stream_infos_get_ident_from_stream(gGui->stream);
 	
 	if(ident) {
-	  if(strcmp(gGui->mmk.ident, ident)) {
-	    if(gGui->mmk.ident)
-	      free(gGui->mmk.ident);
-	    if(gGui->playlist.mmk[gGui->playlist.cur]->ident)
-	      free(gGui->playlist.mmk[gGui->playlist.cur]->ident);
-	    
-	    gGui->mmk.ident = strdup(ident);
-	    gGui->playlist.mmk[gGui->playlist.cur]->ident = strdup(ident);
-	    
-	    video_window_set_mrl(gGui->mmk.ident);
-	    playlist_mrlident_toggle();
-	    panel_update_mrl_display();
+	  if(gGui->playlist.num && gGui->playlist.cur >= 0 && gGui->playlist.mmk &&
+	     gGui->playlist.mmk[gGui->playlist.cur]) {
+	    if(strcmp(gGui->mmk.ident, ident)) {
+	      if(gGui->mmk.ident)
+		free(gGui->mmk.ident);
+	      if(gGui->playlist.mmk[gGui->playlist.cur]->ident)
+		free(gGui->playlist.mmk[gGui->playlist.cur]->ident);
+	      
+	      gGui->mmk.ident = strdup(ident);
+	      gGui->playlist.mmk[gGui->playlist.cur]->ident = strdup(ident);
+	      
+	      video_window_set_mrl(gGui->mmk.ident);
+	      playlist_mrlident_toggle();
+	      panel_update_mrl_display();
+	    }
 	  }
 	  free(ident);
 	}
@@ -462,44 +463,45 @@ static void *slider_loop(void *dummy) {
 	      
 	    }
 	  }  
+	}
+
+	if(gGui->logo_mode == 0) {
 	  
-	  if(gGui->logo_mode == 0) {
+	  if(panel_is_visible()) {
+
+	    panel_update_runtime_display();
 	    
-	    if(panel_is_visible()) {
+	    if(xitk_is_widget_enabled(panel->playback_widgets.slider_play)) {
+	      if(pos >= 0)
+		xitk_slider_set_pos(panel->playback_widgets.slider_play, pos);
+	    }
+	    
+	    if(!(i % 20)) {
+	      int level = 0;
 	      
-	      panel_update_runtime_display();
+	      panel_update_channel_display();
 	      
-	      if(xitk_is_widget_enabled(panel->playback_widgets.slider_play)) {
-		if(pos >= 0)
-		  xitk_slider_set_pos(panel->playback_widgets.slider_play, pos);
+	      if((gGui->mixer.method == SOUND_CARD_MIXER) &&
+		 (gGui->mixer.caps & MIXER_CAP_VOL)) {
+		level = gGui->mixer.volume_level = 
+		  xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
 	      }
+	      else if(gGui->mixer.method == SOFTWARE_MIXER) {
+		level = gGui->mixer.amp_level = 
+		  xine_get_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL);
+	      }
 	      
-	      if(!(i % 20)) {
-		int level = 0;
-		
-		panel_update_channel_display();
-		
-		if((gGui->mixer.method == SOUND_CARD_MIXER) &&
-		   (gGui->mixer.caps & MIXER_CAP_VOL)) {
-		  level = gGui->mixer.volume_level = 
-		    xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
-		}
-		else if(gGui->mixer.method == SOFTWARE_MIXER) {
-		  level = gGui->mixer.amp_level = 
-		    xine_get_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL);
-		}
-		
-		xitk_slider_set_pos(panel->mixer.slider, level);
-		panel_check_mute();
-		
-		i = 0;
-	      }
+	      xitk_slider_set_pos(panel->mixer.slider, level);
+	      panel_check_mute();
 	      
+	      i = 0;
 	    }
 	    
-	    if(stream_infos_is_visible() && gGui->stream_info_auto_update)
-	      stream_infos_update_infos();
 	  }
+	  
+	  if(stream_infos_is_visible() && gGui->stream_info_auto_update)
+	    stream_infos_update_infos();
+
 	}
       }
     }
@@ -574,8 +576,11 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
   if(((!panel->visible || !visible) && !tvset_is_visible()) || (visible && tvset_is_visible()))
     tvset_toggle_visibility(NULL, NULL);
 
-  if(((!panel->visible || !visible) && !pplugin_is_visible()) || (visible && pplugin_is_visible()))
-    pplugin_toggle_visibility(NULL, NULL);
+  if(((!panel->visible || !visible) && !vpplugin_is_visible()) || (visible && vpplugin_is_visible()))
+    vpplugin_toggle_visibility(NULL, NULL);
+
+  if(((!panel->visible || !visible) && !applugin_is_visible()) || (visible && applugin_is_visible()))
+    applugin_toggle_visibility(NULL, NULL);
 
   if(((!panel->visible || !visible) && !help_is_visible()) || (visible && help_is_visible()))
     help_toggle_visibility(NULL, NULL);
@@ -595,7 +600,9 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
 	panel->visible = 0;
 	XUnmapWindow (gGui->display, gGui->panel_window);
 	XSync(gGui->display, False);
+	XUnlockDisplay(gGui->display);
 	xitk_hide_widgets(panel->widget_list);
+	XLockDisplay(gGui->display);
       }
     }
     else {
@@ -617,6 +624,7 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
   else {
 
     panel->visible = 1;
+    gGui->nongui_error_msg = NULL;
     xitk_show_widgets(panel->widget_list);
     
     XLockDisplay(gGui->display);
@@ -624,9 +632,10 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
     XRaiseWindow(gGui->display, gGui->panel_window); 
     XMapWindow(gGui->display, gGui->panel_window); 
     if(!gGui->use_root_window && gGui->video_display == gGui->display)
-      XSetTransientForHint (gGui->display,  gGui->panel_window, gGui->video_window);
+      XSetTransientForHint (gGui->display, gGui->panel_window, gGui->video_window);
     XUnlockDisplay (gGui->display);
 
+    wait_for_window_visible(gGui->display, gGui->panel_window);
     layer_above_video(gGui->panel_window);
      
     if(gGui->cursor_grabbed) {
@@ -680,6 +689,7 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
 	  xitk_slider_set_pos(panel->playback_widgets.slider_play, pos);
 	panel_update_runtime_display();
       }
+
       panel_update_mrl_display();
     }
     
@@ -800,9 +810,9 @@ static void panel_spu_lang_list(xitk_widget_t *w, void *data) {
   int wx, wy;
 
   xitk_get_window_position(gGui->display, gGui->panel_window, &wx, &wy, NULL, NULL);
-  xitk_get_widget_pos(panel->audiochan_label, &x, &y);
+  xitk_get_widget_pos(panel->spuid_label, &x, &y);
   x += wx;
-  y += (wy + xitk_get_widget_height(panel->audiochan_label));
+  y += (wy + xitk_get_widget_height(panel->spuid_label));
   spu_lang_menu(panel->widget_list, x, y);
 }
 /*
@@ -903,6 +913,34 @@ static void panel_handle_event(XEvent *event, void *data) {
     XUnlockDisplay(gGui->display);
     break;
 
+  case MapNotify:
+    /* When panel becomes visible by any means,               */
+    /* all other hidden GUI windows shall also become visible */
+    if(!playlist_is_visible())
+      playlist_toggle_visibility(NULL, NULL);
+    if(!control_is_visible())
+      control_toggle_visibility(NULL, NULL);
+    if(!mrl_browser_is_visible())
+      mrl_browser_toggle_visibility(NULL, NULL);
+    if(!setup_is_visible())
+      setup_toggle_visibility(NULL, NULL);
+    if(!viewlog_is_visible())
+      viewlog_toggle_visibility(NULL, NULL);
+    if(!kbedit_is_visible())
+      kbedit_toggle_visibility(NULL, NULL);
+    if(!event_sender_is_visible())
+      event_sender_toggle_visibility(NULL, NULL);
+    if(!stream_infos_is_visible())
+      stream_infos_toggle_visibility(NULL, NULL);
+    if(!tvset_is_visible())
+      tvset_toggle_visibility(NULL, NULL);
+    if(!vpplugin_is_visible())
+      vpplugin_toggle_visibility(NULL, NULL);
+    if(!applugin_is_visible())
+      applugin_toggle_visibility(NULL, NULL);
+    if(!help_is_visible())
+      help_toggle_visibility(NULL, NULL);
+    break;
   }
   
 }
@@ -1455,14 +1493,14 @@ void panel_init (void) {
   panel->tips.enable = xine_config_register_bool(gGui->xine, "gui.tips_visible", 
 						 1,
 						 _("gui tips visibility"), 
-						 CONFIG_NO_HELP,
+						 _("If disabled, no tooltips are shown."), 
 						 CONFIG_LEVEL_ADV,
 						 panel_enable_tips_cb,
 						 CONFIG_NO_DATA);
   panel->tips.timeout = (unsigned long) xine_config_register_num(gGui->xine, "gui.tips_timeout", 
-								 500,
-								 _("tips timeout (ms)"), 
-								 CONFIG_NO_HELP,
+								 5000,
+								 _("Tips timeout (ms)"), 
+								 _("Persistence time of tooltips, in milliseconds."), 
 								 CONFIG_LEVEL_ADV,
 								 panel_timeout_tips_cb, 
 								 CONFIG_NO_DATA);
