@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: playlist.c,v 1.147 2004/04/14 22:38:30 f1rmb Exp $
+ * $Id: playlist.c,v 1.160 2005/02/06 02:09:42 miguelfreitas Exp $
  *
  * Playlist Editor
  *
@@ -66,9 +66,11 @@ typedef struct {
   char                **playlist_mrls;
   char                **playlist_idents;
   int                   playlist_len;
+
 } _playlist_t;
 
-static _playlist_t   *playlist;
+static filebrowser_t   *load_fb = NULL, *save_fb = NULL;
+static _playlist_t     *playlist;
 
 void playlist_handle_event(XEvent *event, void *data);
 
@@ -130,8 +132,8 @@ static void _playlist_free_playlists(void) {
       playlist->playlist_len--;
     }
     
-    free(playlist->playlist_mrls);
-    free(playlist->playlist_idents);
+    SAFE_FREE(playlist->playlist_mrls);
+    SAFE_FREE(playlist->playlist_idents);
     playlist->playlist_len = 0;
   }
 }
@@ -170,51 +172,22 @@ static void _playlist_handle_selection(xitk_widget_t *w, void *data, int selecte
 }
 
 static void _playlist_xine_play(void) {
-  const mediamark_t *mmk;
+  gui_set_current_mmk(mediamark_get_current_mmk());
 
-  if((mmk = mediamark_get_current_mmk()) != NULL) {
+  if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
+                             gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset,
+                             !mediamark_have_alternates(&(gGui->mmk)))) {
     
-    if((xine_get_status(gGui->stream) != XINE_STATUS_STOP)) {
-      gGui->ignore_next = 1;
-      xine_stop(gGui->stream);
-      gGui->ignore_next = 0;
-    }
-    
-    if(!is_playback_widgets_enabled())
-      enable_playback_controls(1);
-
-    if(mrl_look_like_playlist(mmk->mrl)) {
-      if(mediamark_concat_mediamarks(mmk->mrl)) {
-	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
-	mmk = mediamark_get_current_mmk();
-	playlist_update_playlist();
-      }
-    }
-
-    if(!xine_open(gGui->stream, mmk->mrl)) {
-      gui_handle_xine_error(gGui->stream, mmk->mrl);
-      enable_playback_controls(0);
+    if(!mediamark_have_alternates(&(gGui->mmk)) ||
+        !gui_open_and_play_alternates(&(gGui->mmk), gGui->mmk.sub)) {
+  
+      if(mediamark_all_played() && (gGui->actions_on_start[0] == ACTID_QUIT))
+	gui_exit(NULL, NULL);
+      
       gui_display_logo();
-      return;
-    }
-
-    if(mmk->sub) {
-      if(xine_open(gGui->spu_stream, mmk->sub))
-	xine_stream_master_slave(gGui->stream, 
-				 gGui->spu_stream, XINE_MASTER_SLAVE_PLAY | XINE_MASTER_SLAVE_STOP);
-    }
-    else
-      xine_close(gGui->spu_stream);
-    
-    if(!gui_xine_play(gGui->stream, 0, mmk->start, 0)) {
-      enable_playback_controls(0);
-      gui_display_logo();
-      return;
     }
   }
-
 }
-
 /*
  * Start playing an MRL
  */
@@ -224,9 +197,7 @@ void playlist_play_current(xitk_widget_t *w, void *data) {
   j = xitk_browser_get_current_selected(playlist->playlist);
 
   if((j >= 0) && (gGui->playlist.mmk[j]->mrl != NULL)) {
-    
     gGui->playlist.cur = j;
-    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
     _playlist_xine_play();
     xitk_browser_release_all_buttons(playlist->playlist);
   }
@@ -237,58 +208,69 @@ void playlist_play_current(xitk_widget_t *w, void *data) {
 static void _playlist_play_on_dbl_click(xitk_widget_t *w, void *data, int selected) {
   
   if(gGui->playlist.mmk[selected]->mrl != NULL) {
-    
     gGui->playlist.cur = selected;
-    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
     _playlist_xine_play();
     xitk_browser_release_all_buttons(playlist->playlist);
   }
 }
 
 /*
- * Delete selected MRL
+ * Delete a given entry from playlist
  */
-void playlist_delete_current(xitk_widget_t *w, void *data) {
-  int i, j;
-  
-  mmk_editor_end();
+void playlist_delete_entry(int j) {
+  int i;
+
+  if(j  >= 0) {
 
-  if((j = xitk_browser_get_current_selected(playlist->playlist)) >= 0) {
-    
     if((gGui->playlist.cur == j) && ((xine_get_status(gGui->stream) != XINE_STATUS_STOP)))
       gui_stop(NULL, NULL);
-    
+
     mediamark_free_entry(j);
-    
+
     for(i = j; i < gGui->playlist.num; i++)
       gGui->playlist.mmk[i] = gGui->playlist.mmk[i + 1];
-    
-    gGui->playlist.mmk = (mediamark_t **) 
-      realloc(gGui->playlist.mmk, sizeof(mediamark_t *) * (gGui->playlist.num + 2));
-    
+
+    gGui->playlist.mmk = (mediamark_t **) realloc(gGui->playlist.mmk, sizeof(mediamark_t *) * (gGui->playlist.num + 2));
+
     gGui->playlist.mmk[gGui->playlist.num] = NULL;
-    
-    gGui->playlist.cur = 0;
-    
+
+
     playlist_update_playlist();
-    
-    if(gGui->playlist.num)
-      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+
+    if(gGui->playlist.num) {
+      gui_set_current_mmk(mediamark_get_current_mmk());
+      gGui->playlist.cur = 0;
+    }
     else {
-      
+
+      gGui->playlist.cur = -1;
+
       if(is_playback_widgets_enabled())
 	enable_playback_controls(0);
-      
+
       if(xine_get_status(gGui->stream) != XINE_STATUS_STOP)
 	gui_stop(NULL, NULL);
-      
-      gui_set_current_mrl(NULL);
+
+      gui_set_current_mmk(NULL);
       xitk_inputtext_change_text(playlist->winput, NULL);
     }
   }
 
 }
 
+/*
+ * Delete selected MRL
+ */
+void playlist_delete_current(xitk_widget_t *w, void *data) {
+  int j;
+  
+  mmk_editor_end();
+
+  j = xitk_browser_get_current_selected(playlist->playlist);
+
+  playlist_delete_entry(j);
+}
+
 /*
  * Delete all MRLs
  */
@@ -308,7 +290,7 @@ void playlist_delete_all(xitk_widget_t *w, void *data) {
   if(playlist)
     xitk_browser_release_all_buttons(playlist->playlist);
 
-  gui_set_current_mrl(NULL);
+  gui_set_current_mmk(NULL);
   enable_playback_controls(0);
 }
 
@@ -371,7 +353,7 @@ static void _playlist_load_callback(filebrowser_t *fb) {
 
   if((file = filebrowser_get_full_filename(fb)) != NULL) {
     mediamark_load_mediamarks(file);
-    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+    gui_set_current_mmk(mediamark_get_current_mmk());
     playlist_update_playlist();
 
     if((xine_get_status(gGui->stream) == XINE_STATUS_PLAY))
@@ -382,24 +364,34 @@ static void _playlist_load_callback(filebrowser_t *fb) {
 
     free(file);
   }
+  load_fb = NULL;
   playlist_reactivate();
 }
 static void _playlist_cancel_callback(filebrowser_t *fb) {
+  if(fb == load_fb)
+    load_fb = NULL;
+  else if(fb == save_fb)
+    save_fb = NULL;
+
   playlist_reactivate();
 }
 void playlist_load_playlist(xitk_widget_t *w, void *data) {
   filebrowser_callback_button_t  cbb[2];
   char                           buffer[XITK_PATH_MAX + XITK_NAME_MAX + 1];
 
-  sprintf(buffer, "%s/.xine/playlist.tox", xine_get_homedir());
-  
-  cbb[0].label = _("Load");
-  cbb[0].callback = _playlist_load_callback;
-  cbb[0].need_a_file = 1;
-  cbb[1].callback = _playlist_cancel_callback;
-  
-  playlist_deactivate();
-  (void *) create_filebrowser(_("Load a playlist"), buffer, &cbb[0], NULL, &cbb[1]);
+  if(load_fb)
+    filebrowser_raise_window(load_fb);
+  else {
+    snprintf(buffer, sizeof(buffer), "%s%s", xine_get_homedir(), "/.xine/playlist.tox");
+    
+    cbb[0].label = _("Load");
+    cbb[0].callback = _playlist_load_callback;
+    cbb[0].need_a_file = 1;
+    cbb[1].callback = _playlist_cancel_callback;
+    
+    playlist_deactivate();
+    load_fb = create_filebrowser(_("Load a playlist"), buffer, hidden_file_cb, &cbb[0], NULL, &cbb[1]);
+  }
 }
 
 /*
@@ -412,6 +404,7 @@ static void _playlist_save_callback(filebrowser_t *fb) {
     mediamark_save_mediamarks(file);
     free(file);
   }
+  save_fb = NULL;
   playlist_reactivate();
 }
 void playlist_save_playlist(xitk_widget_t *w, void *data) {
@@ -419,15 +412,19 @@ void playlist_save_playlist(xitk_widget_t *w, void *data) {
   char                           buffer[XITK_PATH_MAX + XITK_NAME_MAX + 1];
 
   if(gGui->playlist.num) {
-    sprintf(buffer, "%s/.xine/playlist.tox", xine_get_homedir());
-    
-    cbb[0].label = _("Save");
-    cbb[0].callback = _playlist_save_callback;
-    cbb[0].need_a_file = 1;
-    cbb[1].callback = _playlist_cancel_callback;
-  
-    playlist_deactivate();
-    (void *) create_filebrowser(_("Save a playlist"), buffer, &cbb[0], NULL, &cbb[1]);
+    if(save_fb)
+      filebrowser_raise_window(save_fb);
+    else {
+      snprintf(buffer, sizeof(buffer), "%s%s", xine_get_homedir(), "/.xine/playlist.tox");
+      
+      cbb[0].label = _("Save");
+      cbb[0].callback = _playlist_save_callback;
+      cbb[0].need_a_file = 1;
+      cbb[1].callback = _playlist_cancel_callback;
+      
+      playlist_deactivate();
+      save_fb = create_filebrowser(_("Save a playlist"), buffer, hidden_file_cb, &cbb[0], NULL, &cbb[1]);
+    }
   }
 }
 
@@ -543,7 +540,12 @@ static void _playlist_handle_event(XEvent *event, void *data) {
 
 static void _playlist_apply_cb(void *data) {
   playlist_mrlident_toggle();
-  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+  gui_set_current_mmk(mediamark_get_current_mmk());
+}
+
+void playlist_get_input_focus(void) {
+  if(playlist)
+    try_to_set_input_focus(playlist->window);
 }
 
 void playlist_mmk_editor(void) {
@@ -626,6 +628,11 @@ void playlist_show_tips(int enabled, unsigned long timeout) {
   }
 }
 
+void playlist_update_tips_timeout(unsigned long timeout) {
+  if(playlist)
+    xitk_set_widgets_tips_timeout(playlist->widget_list, timeout);
+}
+
 void playlist_mrlident_toggle(void) {
 
   if(playlist && playlist->visible) {
@@ -657,6 +664,11 @@ void playlist_update_playlist(void) {
  */
 void playlist_exit(xitk_widget_t *w, void *data) {
 
+  if(load_fb)
+    filebrowser_end(load_fb);
+  if(save_fb)
+    filebrowser_end(save_fb);
+
   if(playlist) {
     window_info_t wi;
     
@@ -696,12 +708,13 @@ void playlist_exit(xitk_widget_t *w, void *data) {
     free(playlist->widget_list);
     
     free(playlist);
-    playlist = NULL;  
+    playlist = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
 
-
 /*
  * return 1 if playlist editor is ON
  */
@@ -722,7 +735,7 @@ int playlist_is_visible(void) {
     if(gGui->use_root_window)
       return xitk_is_window_visible(gGui->display, playlist->window);
     else
-      return playlist->visible;
+      return playlist->visible && xitk_is_window_visible(gGui->display, playlist->window);
   }
 
   return 0;
@@ -756,13 +769,8 @@ void playlist_scan_input(xitk_widget_t *w, void *ip) {
 	    playlist_update_playlist();
 	    if(playlist != NULL)
 	      xitk_inputtext_change_text(playlist->winput, NULL);
-	    gGui->playlist.cur = 0;
 	  }
 	  
-	  if(!gGui->playlist.num)
-	    gGui->playlist.cur = 0;
-	  
-
 	  stream = xine_stream_new(gGui->xine, gGui->ao_none, gGui->vo_none);
 	  
 	  for (j = 0; j < num_mrls; j++) {
@@ -773,7 +781,7 @@ void playlist_scan_input(xitk_widget_t *w, void *ip) {
 	      xine_close(stream);
 	    }
 	    
-	    mediamark_add_entry(autoplay_mrls[j], ident ? ident : autoplay_mrls[j], NULL, 0, -1, 0, 0);
+	    mediamark_append_entry(autoplay_mrls[j], ident ? ident : autoplay_mrls[j], NULL, 0, -1, 0, 0);
 	    
 	    if(ident)
 	      free(ident);
@@ -781,8 +789,10 @@ void playlist_scan_input(xitk_widget_t *w, void *ip) {
 
 	  xine_dispose(stream);
 	  
+	  gGui->playlist.cur = gGui->playlist.num ? 0 : -1;
+	  
 	  if(gGui->playlist.cur == 0)
-	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	    gui_set_current_mmk(mediamark_get_current_mmk());
 	  
 	  /* 
 	   * If we're in newbie mode, start playback immediately
@@ -841,8 +851,8 @@ void playlist_update_focused_entry(void) {
   if(playlist != NULL) {
     if(playlist->window) {
       if(playlist->visible && playlist->running && playlist->playlist_len) {
-	char *pa_mrl, *pl_mrl;
-	mediamark_t *mmk = (mediamark_t *)mediamark_get_current_mmk();
+	char        *pa_mrl, *pl_mrl;
+	mediamark_t *mmk = mediamark_get_current_mmk();
 	
 	if(mmk) {
 
@@ -883,7 +893,7 @@ void playlist_update_focused_entry(void) {
 /*
  * Change the current skin.
  */
-void playlist_change_skins(void) {
+void playlist_change_skins(int synthetic) {
   ImlibImage   *new_img, *old_img;
   XSizeHints    hint;
 
@@ -921,7 +931,7 @@ void playlist_change_skins(void) {
 
     XLockDisplay(gGui->display);
     
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       XSetTransientForHint(gGui->display, playlist->window, gGui->video_window);
     
     Imlib_destroy_image(gGui->imlib_data, old_img);
@@ -991,7 +1001,13 @@ void playlist_deinit(void) {
   if(playlist) {
     if(playlist_is_visible())
       playlist_toggle_visibility(NULL, NULL);
-    xitk_unregister_event_handler(&playlist->widget_key);
+    playlist_exit(NULL, NULL);
+  }
+  else {
+    if(load_fb)
+      filebrowser_end(load_fb);
+    if(save_fb)
+      filebrowser_end(save_fb);
   }
 }
 
