@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: setup.c,v 1.94 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: setup.c,v 1.102 2005/04/28 00:57:58 fsck-p Exp $
  *
  *
  */
@@ -29,7 +29,6 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
-#include <X11/cursorfont.h>
 
 #include "common.h"
 
@@ -50,7 +49,7 @@ static char                     *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-
 #define NORMAL_CURS              0
 #define WAIT_CURS                1
 
-#define ADD_FRAME(title) {                                                                      \
+#define ADD_FRAME(title) do {						                        \
     xitk_widget_t       *frame = NULL;                                                          \
     xitk_image_t        *image;                                                                 \
     xitk_image_widget_t  im;                                                                    \
@@ -86,15 +85,15 @@ static char                     *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-
     wt->frame = frame;                                                                          \
     x += 10;                                                                                    \
     y += FRAME_HEIGHT >> 1;                                                                     \
-  }
+  } while(0)
 
-#define ADD_LABEL(widget, cb, data) {                                                           \
+#define ADD_LABEL(widget, cb, data) do {                                                        \
     int            wx, wy, wh, fh;                                                              \
     xitk_font_t   *fs;                                                                          \
     xitk_widget_t *lbl;                                                                         \
     char           _labelkey[strlen(labelkey) + 5];                                             \
                                                                                                 \
-    sprintf(_labelkey, "%s%s", labelkey,                                                        \
+    snprintf(_labelkey, sizeof(_labelkey), "%s%s", labelkey,		                        \
       (!entry->callback_data && !entry->callback) ? " (*)" : "");                               \
                                                                                                 \
     fs = xitk_font_load_font(gGui->display, fontname);                                          \
@@ -110,35 +109,41 @@ static char                     *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-
                            (FRAME_WIDTH - (xitk_get_widget_width(widget) + 35)),                \
                            _labelkey, cb, data);                                                \
     wt->label = lbl;                                                                            \
-  }
+  } while(0)
 
-#define DISABLE_ME(wtriplet) {                                                                  \
+#define DISABLE_ME(wtriplet) do {                                                               \
     if((wtriplet)->frame)                                                                       \
       xitk_disable_and_hide_widget((wtriplet)->frame);                                          \
                                                                                                 \
     if((wtriplet)->label) {                                                                     \
       xitk_disable_and_hide_widget((wtriplet)->label);                                          \
-      xitk_disable_widget_tips((wtriplet)->label);                                              \
+      xitk_disable_widget_tips((wtriplet)->label);		 	                        \
     }                                                                                           \
     if((wtriplet)->widget) {                                                                    \
       xitk_disable_and_hide_widget((wtriplet)->widget);                                         \
-      xitk_disable_widget_tips((wtriplet)->widget);                                             \
+      xitk_disable_widget_tips((wtriplet)->widget);			                        \
     }                                                                                           \
-}
+  } while(0)
 
-#define ENABLE_ME(wtriplet) {                                                                   \
+#define ENABLE_ME(wtriplet) do {                                                                \
     if((wtriplet)->frame)                                                                       \
       xitk_enable_and_show_widget((wtriplet)->frame);                                           \
                                                                                                 \
     if((wtriplet)->label) {                                                                     \
       xitk_enable_and_show_widget((wtriplet)->label);                                           \
-      xitk_enable_widget_tips((wtriplet)->label);                                               \
+      if(panel_get_tips_enable())					                        \
+	xitk_set_widget_tips_timeout((wtriplet)->label, panel_get_tips_timeout());              \
+      else								                        \
+	xitk_disable_widget_tips((wtriplet)->label);			                        \
     }                                                                                           \
     if((wtriplet)->widget) {                                                                    \
       xitk_enable_and_show_widget((wtriplet)->widget);                                          \
-      xitk_enable_widget_tips((wtriplet)->widget);                                              \
+      if(panel_get_tips_enable())					                        \
+	xitk_set_widget_tips_timeout((wtriplet)->widget, panel_get_tips_timeout());             \
+      else								                        \
+	xitk_disable_widget_tips((wtriplet)->widget);			                        \
     }                                                                                           \
-}
+  } while(0)
 
 
 typedef struct {
@@ -170,15 +175,12 @@ typedef struct {
   int                   num_wg;
   int                   first_displayed;
 
-  Cursor                cursor[2];
-
   xitk_register_key_t   kreg;
 
 } _setup_t;
 
 static _setup_t    *setup = NULL;
 
-static void setup_end(xitk_widget_t *, void *);
 static void setup_change_section(xitk_widget_t *, void *, int);
 
 static void add_widget_to_list(xitk_widget_t *w) {
@@ -188,7 +190,7 @@ static void add_widget_to_list(xitk_widget_t *w) {
 /*
  * Leaving setup panel, release memory.
  */
-void setup_exit(xitk_widget_t *w, void *data) {
+static void setup_exit(xitk_widget_t *w, void *data) {
 
   if(setup) {
     window_info_t wi;
@@ -212,8 +214,6 @@ void setup_exit(xitk_widget_t *w, void *data) {
     xitk_list_free(setup->widgets);
     
     XLockDisplay(gGui->display);
-    XFreeCursor(gGui->display, setup->cursor[NORMAL_CURS]);
-    XFreeCursor(gGui->display, setup->cursor[WAIT_CURS]);
     XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(setup->widget_list)));
     XUnlockDisplay(gGui->display);
     
@@ -221,9 +221,26 @@ void setup_exit(xitk_widget_t *w, void *data) {
     
     free(setup);
     setup = NULL;
+
+    try_to_set_input_focus(gGui->video_window);
   }
 }
 
+void setup_show_tips(int enabled, unsigned long timeout) {
+  
+  if(setup) {
+    if(enabled)
+      xitk_set_widgets_tips_timeout(setup->widget_list, timeout);
+    else
+      xitk_disable_widgets_tips(setup->widget_list);
+  }
+}
+
+void setup_update_tips_timeout(unsigned long timeout) {
+  if(setup)
+    xitk_set_widgets_tips_timeout(setup->widget_list, timeout);
+}
+
 /*
  * return 1 if setup panel is ON
  */
@@ -244,7 +261,7 @@ int setup_is_visible(void) {
     if(gGui->use_root_window)
       return xitk_is_window_visible(gGui->display, xitk_window_get_window(setup->xwin));
     else
-      return setup->visible;
+      return setup->visible && xitk_is_window_visible(gGui->display, xitk_window_get_window(setup->xwin));
   }
 
   return 0;
@@ -347,7 +364,7 @@ static void setup_apply(xitk_widget_t *w, void *data) {
 				 ALIGN_CENTER,
 				 _("You changed some configuration value which require"
 				   " to restart xine to take effect."));
-      if(!gGui->use_root_window) {
+      if(!gGui->use_root_window && gGui->video_display == gGui->display) {
 	XLockDisplay(gGui->display);
 	XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
 	XUnlockDisplay(gGui->display);
@@ -360,17 +377,16 @@ static void setup_apply(xitk_widget_t *w, void *data) {
 
 static void setup_set_cursor(int state) {
   if(setup) {
-    XLockDisplay(gGui->display);
-    XDefineCursor(gGui->display, (xitk_window_get_window(setup->xwin)), 
-		  setup->cursor[state]);
-    XSync(gGui->display, False);
-    XUnlockDisplay(gGui->display);
+    if(state == WAIT_CURS)
+      xitk_cursors_define_window_cursor(gGui->display, (xitk_window_get_window(setup->xwin)), xitk_cursor_watch);
+    else
+      xitk_cursors_restore_window_cursor(gGui->display, (xitk_window_get_window(setup->xwin)));
   }
 }
 
 static void setup_ok(xitk_widget_t *w, void *data) {
   setup_apply(w, data);
-  setup_end(w, data);
+  setup_exit(w, data);
 }
 
 /*
@@ -509,6 +525,10 @@ static void setup_handle_event(XEvent *event, void *data) {
       case XK_Escape:
 	setup_exit(NULL, NULL);
 	break;
+
+      default:
+	gui_handle_event(event, data);
+	break;
       }
     }
     break;
@@ -816,53 +836,46 @@ static void setup_section_widgets(int s) {
       labelkey = &entry->key[len+1];
 	
       switch (entry->type) {
-	  
+
+#define _SET_HELP do {						                                           \
+	  xitk_set_widget_tips_and_timeout(setup->wg[setup->num_wg]->widget,                               \
+					   (entry->help) ?  (char *) entry->help : _("No help available"), \
+					   panel_get_tips_timeout());	                                   \
+	  xitk_set_widget_tips_and_timeout(setup->wg[setup->num_wg]->label,                                \
+					   (entry->help) ?  (char *) entry->help : _("No help available"), \
+					   panel_get_tips_timeout());	                                   \
+	} while(0)
       case XINE_CONFIG_TYPE_RANGE: /* slider */
 	setup->wg[setup->num_wg] = setup_add_slider (entry->description, labelkey, x, y, entry);
-	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			     (entry->help) ?  (char *) entry->help : _("No help available"));
-	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	_SET_HELP;
 	DISABLE_ME(setup->wg[setup->num_wg]);
 	setup->num_wg++;
 	break;
-	  
+	
       case XINE_CONFIG_TYPE_STRING:
 	setup->wg[setup->num_wg] = setup_add_inputtext (entry->description, labelkey, x, y, entry);
-	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			     (entry->help) ?  (char *) entry->help : _("No help available"));
-	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	_SET_HELP;
 	DISABLE_ME(setup->wg[setup->num_wg]);
 	setup->num_wg++;
 	break;
-	  
+	
       case XINE_CONFIG_TYPE_ENUM:
 	setup->wg[setup->num_wg] = setup_add_combo (entry->description, labelkey, x, y, entry);
-	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			     (entry->help) ?  (char *) entry->help : _("No help available"));
-	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	_SET_HELP;
 	DISABLE_ME(setup->wg[setup->num_wg]);
 	setup->num_wg++;
 	break;
-	  
+	
       case XINE_CONFIG_TYPE_NUM:
 	setup->wg[setup->num_wg] = setup_add_inputnum (entry->description, labelkey, x, y, entry);
-	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			     (entry->help) ?  (char *) entry->help : _("No help available"));
-	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	_SET_HELP;
 	DISABLE_ME(setup->wg[setup->num_wg]);
 	setup->num_wg++;
 	break;
-	  
+	
       case XINE_CONFIG_TYPE_BOOL:
 	setup->wg[setup->num_wg] = setup_add_checkbox (entry->description, labelkey, x, y, entry);
-	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			     (entry->help) ?  (char *) entry->help : _("No help available"));
-	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	_SET_HELP;
 	DISABLE_ME(setup->wg[setup->num_wg]);
 	setup->num_wg++;
 	break;
@@ -1026,7 +1039,7 @@ static void setup_sections (void) {
 /*
  *
  */
-static void setup_end(xitk_widget_t *w, void *data) {
+void setup_end(void) {
   setup_exit(NULL, NULL);
 }
 
@@ -1086,8 +1099,6 @@ void setup_panel(void) {
   XLockDisplay (gGui->display);
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(setup->xwin)), None, None);
-  setup->cursor[NORMAL_CURS] = XCreateFontCursor(gGui->display, XC_left_ptr);
-  setup->cursor[WAIT_CURS] = XCreateFontCursor(gGui->display, XC_watch);
   XUnlockDisplay (gGui->display);
 
   setup->widget_list                = xitk_widget_list_new();
@@ -1173,7 +1184,7 @@ void setup_panel(void) {
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Close");
   lb.align             = ALIGN_CENTER;
-  lb.callback          = setup_end; 
+  lb.callback          = setup_exit; 
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
@@ -1182,6 +1193,7 @@ void setup_panel(void) {
 				       (x * 3) + (100 * 2), WINDOW_HEIGHT - 40, 100, 23,
 				       "Black", "Black", "White", tabsfontname)));
   xitk_enable_and_show_widget(w);
+  setup_show_tips(panel_get_tips_enable(), panel_get_tips_timeout());
   
   setup->kreg = xitk_register_event_handler("setup", 
 					    (xitk_window_get_window(setup->xwin)),
