@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  *
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: snapshot.c,v 1.27 2003/07/14 22:44:09 f1rmb Exp $
+ * $Id: snapshot.c,v 1.30 2005/01/15 22:11:55 fsck-p Exp $
  *
  * Image snapshot related stuff
  *
@@ -92,6 +92,10 @@ struct prvt_image_s {
   uint8_t *y, *u, *v, *yuy2;
   uint8_t *img;
 
+  /* Pointers to allocated mem that has to be freed at the end */
+  uint8_t *scale_image_y, *scale_image_u, *scale_image_v;
+  uint8_t *yuy2_fudge_y, *yuy2_fudge_u, *yuy2_fudge_v;
+
   int u_width, v_width;
   int u_height, v_height;
 
@@ -515,7 +519,7 @@ static void scale_line_15_16 (uint8_t *source, uint8_t *dest,
  done: ;
 }
 
-int scale_image( struct prvt_image_s *image )
+static int scale_image( struct prvt_image_s *image )
 {
   int i;
 
@@ -549,10 +553,13 @@ int scale_image( struct prvt_image_s *image )
   /* allocate new buffer space space for post-scaled line buffers */
   n_y = (uint8_t*)png_malloc( image->struct_ptr, ny_width * image->height );
   if (n_y == 0) return( 0 );
+  image->scale_image_y = n_y;
   n_u = (uint8_t*)png_malloc( image->struct_ptr, nu_width * image->u_height );
   if (n_u == 0) return( 0 );
+  image->scale_image_u = n_u;
   n_v = (uint8_t*)png_malloc( image->struct_ptr, nv_width * image->v_height );
   if (n_v == 0) return( 0 );
+  image->scale_image_v = n_v;
 
   /* set post-scaled line buffer progress pointers */
   ny_p = n_y;
@@ -614,14 +621,17 @@ static int yuy2_fudge( struct prvt_image_s *image )
 {
   image->y = png_malloc( image->struct_ptr, image->height*image->width );
   if ( image->y == NULL ) return( 0 );
+  image->yuy2_fudge_y = image->y;
   memset( image->y, 0, image->height*image->width );
 
   image->u = png_malloc( image->struct_ptr, image->u_height*image->u_width );
   if ( image->u == NULL ) return( 0 );
+  image->yuy2_fudge_u = image->u;
   memset( image->u, 0, image->u_height*image->u_width );
 
   image->v = png_malloc( image->struct_ptr, image->v_height*image->v_width );
   if ( image->v == NULL ) return( 0 );
+  image->yuy2_fudge_v = image->v;
   memset( image->v, 0, image->v_height*image->v_width );
 
   yuy2toyv12( image );
@@ -684,7 +694,7 @@ static char *snap_build_filename(const char *mrl) {
     
     p++;
     
-    sprintf(basename, "%s", p);
+    snprintf(basename, sizeof(basename), "%s", p);
     
     if((ext = strrchr(basename, '.')) != NULL)
       *ext = '\0';
@@ -692,7 +702,7 @@ static char *snap_build_filename(const char *mrl) {
   }
 
   for(i = 1;; i++) {
-    sprintf(buffer, "%s/%s-%d.png", gGui->snapshot_location, basename, i);
+    snprintf(buffer, sizeof(buffer), "%s/%s-%d%s", gGui->snapshot_location, basename, i, ".png");
     if(((stat(buffer, &sstat)) == -1) && (errno == ENOENT))
       break;
   }
@@ -712,6 +722,8 @@ static int prvt_image_alloc( struct prvt_image_s **image, int imgsize )
     free(*image);
     return 0;
   }
+  (*image)->scale_image_y = (*image)->scale_image_u = (*image)->scale_image_v = NULL;
+  (*image)->yuy2_fudge_y = (*image)->yuy2_fudge_u = (*image)->yuy2_fudge_v = NULL;
   
   return( 1 );
 }
@@ -729,9 +741,17 @@ static void prvt_image_free( struct prvt_image_s **image )
    
   rgb_free ( image_p );
 
+  if (image_p->scale_image_y) png_free( image_p->struct_ptr, image_p->scale_image_y );
+  if (image_p->scale_image_u) png_free( image_p->struct_ptr, image_p->scale_image_u );
+  if (image_p->scale_image_v) png_free( image_p->struct_ptr, image_p->scale_image_v );
+  if (image_p->yuy2_fudge_y)  png_free( image_p->struct_ptr, image_p->yuy2_fudge_y );
+  if (image_p->yuy2_fudge_u)  png_free( image_p->struct_ptr, image_p->yuy2_fudge_u );
+  if (image_p->yuy2_fudge_v)  png_free( image_p->struct_ptr, image_p->yuy2_fudge_v );
+
   if (image_p->info_ptr)   png_destroy_info_struct (  image_p->struct_ptr, &image_p->info_ptr );
   if (image_p->struct_ptr) png_destroy_write_struct( &image_p->struct_ptr, (png_infopp)NULL );
   if (image_p->fp)         fclose( image_p->fp );
+
   free(image_p);
 }
 
@@ -824,7 +844,7 @@ static void user_error_fn(png_structp png_ptr, png_const_charp error_msg)
     char uerror[4096]; 
 
     memset(&uerror, 0, sizeof(uerror));
-    sprintf(uerror, _("Error: %s\n"), error_msg);
+    snprintf(uerror, sizeof(uerror), "%s%s\n", _("Error: "), error_msg);
     error_msg_cb(msg_cb_data, uerror);
   }
 }
@@ -835,7 +855,7 @@ static void user_warning_fn(png_structp png_ptr, png_const_charp warning_msg)
     char uerror[4096];
     
     memset(&uerror, 0, sizeof(uerror));
-    sprintf(uerror, _("Error: %s\n"), warning_msg);
+    snprintf(uerror, sizeof(uerror), "%s%s\n", _("Error: "), warning_msg);
     error_msg_cb(msg_cb_data, uerror);
   }
 }
@@ -887,7 +907,7 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
       char umessage[4096];
       
       memset(&umessage, 0, sizeof(umessage));
-      sprintf(umessage, _("Wrong image size: %dx%d. Snapshot aborted.\n"), width, height);
+      snprintf(umessage, sizeof(umessage), "%s%dx%d%s\n", _("Wrong image size: "), width, height, _(". Snapshot aborted."));
       error_msg_cb(msg_cb_data, umessage);
     }
     return;
@@ -1024,7 +1044,7 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
       char umessage[4096];
       
       memset(&umessage, 0, sizeof(umessage));
-      sprintf(umessage, _("File open failed (%s)\n"), image->file_name);
+      snprintf(umessage, sizeof(umessage), "%s (%s)\n", _("File open failed"), image->file_name);
       error_msg_cb(msg_cb_data, umessage);
     }
     prvt_image_free( &image );
@@ -1093,6 +1113,7 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
 #endif
     if ( yuy2_fudge( image ) == 0 ) {
       error_msg_cb(msg_cb_data, _("Error: yuy2_fudge failed\n"));
+      prvt_image_free( &image );
       return;
     }
   }
@@ -1204,7 +1225,7 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
   if(info_msg_cb) {
     char umessage[4096];
     memset(&umessage, 0, sizeof(umessage));
-    sprintf(umessage, _("File '%s' written.\n"), image->file_name);
+    snprintf(umessage, sizeof(umessage), "%s%s\n", _("File written: "), image->file_name);
     info_msg_cb(msg_cb_data, umessage);
   }
 
