@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: splash.c,v 1.6 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: splash.c,v 1.8 2004/12/16 21:38:52 f1rmb Exp $
  *
  * gui initialization and top-level event handling stuff
  *
@@ -44,7 +44,7 @@ void splash_create(void) {
   
   if(skin_path && is_a_dir((char *) skin_path)) {
     memset(&skin_splash_image, 0, sizeof(skin_splash_image));
-    sprintf(skin_splash_image, "%s/%s", skin_path, "xine_splash.png");
+    snprintf(skin_splash_image, sizeof(skin_splash_image), "%s/%s", skin_path, "xine_splash.png");
 
     if(is_a_file(skin_splash_image))
       splash_image = skin_splash_image;
@@ -68,8 +68,7 @@ void splash_create(void) {
     change_class_name((xitk_window_get_window(xwin)));
     change_icon((xitk_window_get_window(xwin)));
     
-    xitk_window_change_background(gGui->imlib_data, xwin, 
-				  xim->image->pixmap, xim->width, xim->height);
+    xitk_window_change_background_with_image(gGui->imlib_data, xwin, xim, xim->width, xim->height);
     
     XLockDisplay(gGui->display);
     XRaiseWindow(gGui->display, xitk_window_get_window(xwin)); 
