@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: _xitk.h,v 1.76 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: _xitk.h,v 1.87 2004/12/16 21:35:30 f1rmb Exp $
  *
  *
  */
@@ -47,6 +47,7 @@
 #include <X11/extensions/XShm.h>
 #endif
 #ifdef WITH_XFT
+#include <ft2build.h>
 #include <X11/Xft/Xft.h>
 #endif
 
@@ -64,9 +65,9 @@ typedef struct {
 #include "button.h"
 #include "checkbox.h"
 #include "combo.h"
+#include "cursors.h"
 #include "dnd.h"
 #include "doublebox.h"
-#include "filebrowser.h"
 #include "font.h"
 #include "image.h"
 #include "inputtext.h"
@@ -261,6 +262,7 @@ void xitk_strdupa(char *dest, char *src);
 #define INPUT_MOTION (ExposureMask | ButtonPressMask | ButtonReleaseMask |    \
                       KeyPressMask | KeyReleaseMask | ButtonMotionMask |      \
                       StructureNotifyMask | PropertyChangeMask |              \
+                      FocusChangeMask |					      \
                       LeaveWindowMask | EnterWindowMask | PointerMotionMask)
 
 #define DIRECTION_UP     1
@@ -272,7 +274,7 @@ void xitk_strdupa(char *dest, char *src);
 #define CHECK_STYLE_CHECK 2
 #define CHECK_STYLE_ROUND 3
 
-int xitk_x_error;
+extern int xitk_x_error;
 
 #ifndef _XITK_C_
 
@@ -317,15 +319,23 @@ unsigned long xitk_get_timer_label_animation(void);
 long int xitk_get_timer_dbl_click(void);
 int xitk_get_barstyle_feature(void);
 int xitk_get_checkstyle_feature(void);
+int xitk_get_cursors_feature(void);
 unsigned long xitk_get_warning_foreground(void);
 unsigned long xitk_get_warning_background(void);
 void xitk_modal_window(Window w);
 void xitk_unmodal_window(Window w);
 void xitk_set_current_menu(xitk_widget_t *menu);
 void xitk_unset_current_menu(void);
+int xitk_get_display_width(void);
+int xitk_get_display_height(void);
+XColor xitk_get_black_pixel_color(void);
+unsigned long xitk_get_tips_timeout(void);
+void xitk_set_tips_timeout(unsigned long timeout);
 
+int xitk_is_running(void);
 int xitk_skin_get_direction(xitk_skin_config_t *, const char *);
 int xitk_skin_get_visibility(xitk_skin_config_t *, const char *);
+int xitk_skin_get_printability(xitk_skin_config_t *, const char *);
 int xitk_skin_get_enability(xitk_skin_config_t *, const char *);
 int xitk_skin_get_coord_x(xitk_skin_config_t *, const char *);
 int xitk_skin_get_coord_y(xitk_skin_config_t *, const char *);
@@ -372,7 +382,9 @@ typedef struct xitk_dialog_s xitk_dialog_t;
 
 struct xitk_window_s {
   Window                    window;
+  Window                    win_parent;
   xitk_pixmap_t            *background;
+  xitk_pixmap_t            *background_mask;
   int                       width;
   int                       height;
   xitk_dialog_t            *parent;
@@ -389,6 +401,8 @@ struct xitk_dialog_s {
   xitk_widget_t          *wyes;
   xitk_widget_t          *wno;
   xitk_widget_t          *wcancel;
+  xitk_widget_t          *checkbox;
+  xitk_widget_t          *checkbox_label;
 
   xitk_widget_t          *default_button;
 
