@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: doublebox.c,v 1.3 2004/04/13 19:55:49 f1rmb Exp $
+ * $Id: doublebox.c,v 1.5 2004/07/21 19:56:17 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -122,6 +122,17 @@ static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
   }
 }
 
+static void tips_timeout(xitk_widget_t *w, unsigned long timeout) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_DOUBLEBOX) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
+    doublebox_private_data_t *private_data = (doublebox_private_data_t *) w->private_data;
+    
+    private_data->input_widget->tips_timeout = timeout;
+    private_data->more_widget->tips_timeout = timeout;
+    private_data->less_widget->tips_timeout = timeout;
+  }
+}
+
 static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_result_t *result) {
   int retval = 0;
   
@@ -138,6 +149,9 @@ static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_re
   case WIDGET_EVENT_ENABLE:
     enability(w);
     break;
+  case WIDGET_EVENT_TIPS_TIMEOUT:
+    tips_timeout(w, event->tips_timeout);
+    break;
   }
   
   return retval;
@@ -157,7 +171,7 @@ static void doublebox_change_value(xitk_widget_t *x, void *data, char *string) {
     private_data->value = strtod(string, &string);
     
     memset(&buf, 0, sizeof(buf));
-    snprintf(buf, 256, "%e", private_data->value);
+    snprintf(buf, sizeof(buf), "%e", private_data->value);
     xitk_inputtext_change_text(private_data->input_widget, buf);
     if(private_data->force_value == 0)
       if(private_data->callback)
@@ -176,7 +190,7 @@ void xitk_doublebox_set_value(xitk_widget_t *w, double value) {
     char                      buf[256];
 
     memset(&buf, 0, sizeof(buf));
-    snprintf(buf, 256, "%e", value);
+    snprintf(buf, sizeof(buf), "%e", value);
     private_data->force_value = 1;
     doublebox_change_value(NULL, (void*)w, buf);
     private_data->force_value = 0;
@@ -315,7 +329,7 @@ xitk_widget_t *xitk_noskin_doublebox_create(xitk_widget_list_t *wl,
     xitk_image_t *wimage;
 
     memset(&buf, 0, sizeof(buf));
-    snprintf(buf, 256, "%e", ib->value);
+    snprintf(buf, sizeof(buf), "%e", ib->value);
 
     inp.skin_element_name = NULL;
     inp.text              = buf; 
