@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: font.c,v 1.36 2004/04/16 12:29:40 f1rmb Exp $
+ * $Id: font.c,v 1.41 2005/01/06 17:46:56 valtri Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -182,6 +182,7 @@ static lang_locale_t lang_locales[] = {
   { "it_IT",    "iso88591",   NULL       },
   { "it_IT",    "iso885915",  "euro"     },
   { "iw_IL",    "iso88598",   NULL       },
+  { "ja_JP",    "utf8",       NULL       },
   { "ja_JP",    "eucjp",      NULL       },
   { "ja_JP",    "ujis",       NULL       },
   { "japanese", "euc",        NULL       },
@@ -313,7 +314,7 @@ static int xitk_font_guess_error(XFontSet fs, char *name, char **missing, int co
 # endif
 
 /* convert a -*-* .. style font description into something Xft can digest */
-char * xitk_font_core_string_to_xft( char * old_name) {
+static char * xitk_font_core_string_to_xft( char * old_name) {
   static char new_name[255];
 
   if( strncmp(old_name, "-*", 2) == 0 || strncmp(old_name, "*-", 2) == 0
@@ -341,16 +342,16 @@ char * xitk_font_core_string_to_xft( char * old_name) {
       int  sz = strtol(psize, &psize, 10);
 
       sz -= 2;
-      sprintf( size , "%i", sz );
+      snprintf(size , sizeof(size), "%i", sz);
     }
     else
-      sprintf(size, "%s", psize);
+      snprintf(size, sizeof(size), "%s", psize);
     
     if( strcmp( style , "bold" ) != 0 )
-      snprintf( new_name, 255, "%s-%s", font, size );
+      snprintf( new_name, sizeof(new_name), "%s-%s", font, size );
     else
-      snprintf( new_name, 255, "%s-%s:%s", font, size, style );
-
+      snprintf( new_name, sizeof(new_name), "%s-%s:%s", font, size, style );
+ 
     return new_name;
   }
 
