@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrlbrowser.c,v 1.81 2004/04/13 19:55:50 f1rmb Exp $
+ * $Id: mrlbrowser.c,v 1.85 2005/04/28 00:57:58 fsck-p Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -96,10 +96,8 @@ static void _duplicate_mrl_filters(mrlbrowser_private_data_t *private_data,
     for(i = 0; 
 	mrl_filters[i] && (mrl_filters[i]->name && mrl_filters[i]->ending); i++) {
 
-      private_data->mrl_filters            = (xitk_mrlbrowser_filter_t **)
-	realloc(private_data->mrl_filters, sizeof(xitk_mrlbrowser_filter_t *) * (i + 2));
-      private_data->mrl_filters[i]         = (xitk_mrlbrowser_filter_t *) 
-	xitk_xmalloc(sizeof(xitk_mrlbrowser_filter_t));
+      private_data->mrl_filters            = (xitk_mrlbrowser_filter_t **) realloc(private_data->mrl_filters, sizeof(xitk_mrlbrowser_filter_t *) * (i + 2));
+      private_data->mrl_filters[i]         = (xitk_mrlbrowser_filter_t *) xitk_xmalloc(sizeof(xitk_mrlbrowser_filter_t));
       private_data->mrl_filters[i]->name   = strdup(mrl_filters[i]->name);
       private_data->mrl_filters[i]->ending = strdup(mrl_filters[i]->ending);
     }
@@ -151,10 +149,9 @@ static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_re
 static void update_current_origin(mrlbrowser_private_data_t *private_data) {
 
   if(private_data->mc->mrls[0] && private_data->mc->mrls[0]->origin)
-    sprintf(private_data->current_origin, "%s", 
-	    private_data->mc->mrls[0]->origin);
+    snprintf(private_data->current_origin, sizeof(private_data->current_origin), "%s", private_data->mc->mrls[0]->origin);
   else
-    sprintf(private_data->current_origin, "%s", "");
+    snprintf(private_data->current_origin, sizeof(private_data->current_origin), "%s", "");
   
   xitk_label_change_label (private_data->widget_origin, 
 			   private_data->current_origin);
@@ -193,7 +190,6 @@ static void mrlbrowser_filter_mrls(mrlbrowser_private_data_t *private_data) {
   if(private_data->filter_selected) {
     char *filter_ends;
     char *p, *m;
-    char *ending;
 
     private_data->mc->mrls_to_disp = 0;
 
@@ -211,9 +207,9 @@ static void mrlbrowser_filter_mrls(mrlbrowser_private_data_t *private_data) {
 	private_data->mc->mrls_to_disp++;
       }
       else {
-	ending = strrchr(private_data->mc->mrls[i]->mrl, '.');
-	
-	if(ending) {
+	char *ending;
+
+	if((ending = strrchr(private_data->mc->mrls[i]->mrl, '.'))) {
 
 	  while((m = xitk_strsep(&filter_ends, ",")) != NULL) {
 	    
@@ -279,14 +275,8 @@ static void mrlbrowser_create_enlighted_entries(mrlbrowser_private_data_t *priva
     
     if(private_data->mc->filtered_mrls[i]->type & XINE_MRL_TYPE_file_symlink) {
 
-      if(private_data->mc->mrls_disp[i])
-	private_data->mc->mrls_disp[i] = (char *) 
-	  realloc(private_data->mc->mrls_disp[i], strlen(p) + 4 + 
-		  strlen(private_data->mc->filtered_mrls[i]->link) + 2);
-      else
-	private_data->mc->mrls_disp[i] = (char *) 
-	  xitk_xmalloc(strlen(p) + 4 + 
-		      strlen(private_data->mc->filtered_mrls[i]->link) + 2);
+      private_data->mc->mrls_disp[i] = (char *) realloc(private_data->mc->mrls_disp[i], 
+							strlen(p) + 4 + strlen(private_data->mc->filtered_mrls[i]->link) + 2);
       
       sprintf(private_data->mc->mrls_disp[i], "%s%c -> %s", 
 	      p, get_mrl_marker(private_data->mc->filtered_mrls[i]), 
@@ -294,11 +284,7 @@ static void mrlbrowser_create_enlighted_entries(mrlbrowser_private_data_t *priva
 
     }
     else {
-      if(private_data->mc->mrls_disp[i])
-	private_data->mc->mrls_disp[i] = (char *) 
-	  realloc(private_data->mc->mrls_disp[i], strlen(p) + 2);
-      else
-	private_data->mc->mrls_disp[i] = (char *) xitk_xmalloc(strlen(p) + 2);
+      private_data->mc->mrls_disp[i] = (char *) realloc(private_data->mc->mrls_disp[i], strlen(p) + 2);
       
       sprintf(private_data->mc->mrls_disp[i], "%s%c", 
 	      p, get_mrl_marker(private_data->mc->filtered_mrls[i]));
@@ -347,11 +333,7 @@ static void mrlbrowser_grab_mrls(xitk_widget_t *w, void *data) {
 
   if(lbl) {
     
-    if(private_data->last_mrl_source)
-      private_data->last_mrl_source = (char *)
-	realloc(private_data->last_mrl_source, strlen(lbl) + 1);
-    else
-      private_data->last_mrl_source = (char *) xitk_xmalloc(strlen(lbl) + 1);
+    private_data->last_mrl_source = (char *) realloc(private_data->last_mrl_source, strlen(lbl) + 1);
     
     old_old_src = strdup(private_data->last_mrl_source);
     sprintf(private_data->last_mrl_source, "%s", lbl);
@@ -362,8 +344,7 @@ static void mrlbrowser_grab_mrls(xitk_widget_t *w, void *data) {
 					       private_data->last_mrl_source, 
 					       NULL, &num_mrls);
       if(!mtmp) {
-	private_data->last_mrl_source = (char *)
-	  realloc(private_data->last_mrl_source, strlen(old_old_src) + 1);
+	private_data->last_mrl_source = (char *) realloc(private_data->last_mrl_source, strlen(old_old_src) + 1);
 	sprintf(private_data->last_mrl_source, "%s", old_old_src);
 	return;
       }
@@ -664,7 +645,7 @@ void xitk_mrlbrowser_destroy(xitk_widget_t *w) {
 /*
  * Leaving mrlbrowser.
  */
-void xitk_mrlbrowser_exit(xitk_widget_t *w, void *data) {
+static void xitk_mrlbrowser_exit(xitk_widget_t *w, void *data) {
   mrlbrowser_private_data_t *private_data = ((xitk_widget_t *)data)->private_data;
   
   if(private_data->kill_callback)
@@ -793,7 +774,7 @@ static void mrlbrowser_select_mrl(mrlbrowser_private_data_t *private_data,
   char   buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
   
   memset(&buf, '\0', sizeof(buf));
-  sprintf(buf, "%s", ms->mrl);
+  snprintf(buf, sizeof(buf), "%s", ms->mrl);
   
   if((ms->type & XINE_MRL_TYPE_file) && (ms->type & XINE_MRL_TYPE_file_directory)) {
     char *filename = ms->mrl;
@@ -810,14 +791,14 @@ static void mrlbrowser_select_mrl(mrlbrowser_private_data_t *private_data,
        (filename[strlen(filename) - 2] != '.')) {
       
       memset(&buf, '\0', sizeof(buf));
-      sprintf(buf, "%s", ms->origin);
+      snprintf(buf, sizeof(buf), "%s", ms->origin);
     }
     else if((filename[strlen(filename) - 1] == '.') &&
 	    (filename[strlen(filename) - 2] == '.')) {
       char *p;
       
       memset(&buf, '\0', sizeof(buf));
-      sprintf(buf, "%s", ms->origin);
+      snprintf(buf, sizeof(buf), "%s", ms->origin);
       if(strlen(buf) > 1) { /* not '/' directory */
 	  
 	p = &buf[strlen(buf)-1];
@@ -974,6 +955,9 @@ static void mrlbrowser_handle_event(XEvent *event, void *data) {
       xitk_mrlbrowser_exit(NULL, (void *)private_data->fbWidget);
       break;
       
+    default:
+      gui_handle_event(event, data);
+      break;
     }
   }
   break;
@@ -1221,7 +1205,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   
   memset(&private_data->current_origin, 0, strlen(private_data->current_origin));
   if(mb->origin.cur_origin)
-    sprintf(private_data->current_origin, "%s", mb->origin.cur_origin);
+    snprintf(private_data->current_origin, sizeof(private_data->current_origin), "%s", mb->origin.cur_origin);
 
   if(mb->ip_name.label.label_str) {
 
