@@ -17,17 +17,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: tips.h,v 1.6 2004/04/13 19:55:50 f1rmb Exp $
+ * $Id: tips.h,v 1.9 2005/05/20 23:03:42 fsck-p Exp $
  *
  */
 
 #ifndef HAVE_XITK_TIPS_H
 #define HAVE_XITK_TIPS_H
 
-#define TIPS_TIMEOUT 500
+#define TIPS_TIMEOUT 5000
+
+void xitk_tips_init(Display *display);
+void xitk_tips_deinit(void);
+void xitk_tips_hide_tips(void);
+
+int xitk_tips_show_widget_tips(xitk_widget_t *w);
 
-void xitk_tips_create(xitk_widget_t *w);
-void xitk_tips_tips_kill(xitk_widget_t *w);
 void xitk_tips_set_timeout(xitk_widget_t *w, unsigned long timeout);
 void xitk_tips_set_tips(xitk_widget_t *w, char *str);
 
