@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: widget.c,v 1.58 2004/04/13 19:55:50 f1rmb Exp $
+ * $Id: widget.c,v 1.67 2004/12/22 00:01:53 fsck-p Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -792,6 +792,7 @@ int xitk_paint_widget_list (xitk_widget_list_t *wl) {
       event.type = WIDGET_EVENT_PAINT;
       (void) mywidget->event(mywidget, &event, NULL);
       
+      mywidget->have_focus = FOCUS_LOST;
     }
     else {
       event.type = WIDGET_EVENT_PAINT;
@@ -925,9 +926,13 @@ void xitk_motion_notify_widget_list(xitk_widget_list_t *wl, int x, int y, unsign
     if (wl->widget_under_mouse) {
       
       /* Kill (hide) tips */
-      xitk_tips_tips_kill(wl->widget_under_mouse);
+      xitk_tips_hide_tips();
       
-      if((wl->widget_under_mouse->type & WIDGET_FOCUSABLE) && 
+      if(!(wl->widget_focused && wl->widget_focused == wl->widget_under_mouse &&
+	   (((wl->widget_focused->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) ||
+	    ((wl->widget_focused->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER &&
+	     (wl->widget_focused->type & WIDGET_KEYABLE)))) &&
+	 (wl->widget_under_mouse->type & WIDGET_FOCUSABLE) &&
 	 wl->widget_under_mouse->enable == WIDGET_ENABLE) {
 	
 	event.type  = WIDGET_EVENT_FOCUS;
@@ -947,20 +952,28 @@ void xitk_motion_notify_widget_list(xitk_widget_list_t *wl, int x, int y, unsign
       dump_widget_type(mywidget);
 #endif
       
-      xitk_tips_create(mywidget);
+      if(xitk_tips_show_widget_tips(mywidget)) {
       
-      if((mywidget->type & WIDGET_FOCUSABLE) && mywidget->enable == WIDGET_ENABLE) {
-	event.type  = WIDGET_EVENT_FOCUS;
-	event.focus = FOCUS_MOUSE_IN;
-	(void) mywidget->event(mywidget, &event, NULL);
+	/* Only give focus and paint when tips are accepted, otherwise associated window is invisible. */
+	/* This call may occur from MotionNotify directly after iconifying window.                     */
+	if(!(wl->widget_focused && wl->widget_focused == wl->widget_under_mouse &&
+	     (((wl->widget_focused->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) ||
+	      ((wl->widget_focused->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER &&
+	       (wl->widget_focused->type & WIDGET_KEYABLE)))) &&
+	   (mywidget->type & WIDGET_FOCUSABLE) && mywidget->enable == WIDGET_ENABLE) {
+	  event.type  = WIDGET_EVENT_FOCUS;
+	  event.focus = FOCUS_MOUSE_IN;
+	  (void) mywidget->event(mywidget, &event, NULL);
 
-	event.type  = WIDGET_EVENT_PAINT;
-	(void) mywidget->event(mywidget, &event, NULL);
+	  event.type  = WIDGET_EVENT_PAINT;
+	  (void) mywidget->event(mywidget, &event, NULL);
+	}
       }
     }
   }
   else {
-    if(mywidget && (mywidget->type & WIDGET_TYPE_SLIDER)) {
+    /* Can the following happen here at all ? */
+    if(mywidget && (mywidget->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER) {
       widget_event_result_t   result;
       
       if(state & Button1Mask) {
@@ -992,11 +1005,11 @@ int xitk_click_notify_widget_list (xitk_widget_list_t *wl, int x, int y, int but
   
   mywidget = xitk_get_widget_at (wl, x, y);
 
-  if(mywidget != wl->widget_focused) {
+  if(mywidget != wl->widget_focused && !bUp) {
     if (wl->widget_focused) {
       
       /* Kill (hide) tips */
-      xitk_tips_tips_kill(wl->widget_focused);
+      xitk_tips_hide_tips();
       
       if((wl->widget_focused->type & WIDGET_FOCUSABLE) &&
 	 wl->widget_focused->enable == WIDGET_ENABLE) {
@@ -1051,9 +1064,9 @@ int xitk_click_notify_widget_list (xitk_widget_list_t *wl, int x, int y, int but
     }
     
   }
-  else {
+  else if (!bUp) {
+
     if(wl->widget_under_mouse && mywidget && (wl->widget_under_mouse == mywidget)) {
-      
       if ((mywidget->type & WIDGET_FOCUSABLE) && mywidget->enable == WIDGET_ENABLE) {
 	event.type  = WIDGET_EVENT_FOCUS;
 	event.focus = FOCUS_RECEIVED;
@@ -1065,14 +1078,14 @@ int xitk_click_notify_widget_list (xitk_widget_list_t *wl, int x, int y, int but
   }
   
   if (!bUp) {
-
+    
     wl->widget_pressed = mywidget;
     
     if (mywidget) {
       widget_event_result_t result;
 
       /* Kill (hide) tips */
-      xitk_tips_tips_kill(mywidget);
+      xitk_tips_hide_tips();
       
       if((mywidget->type & WIDGET_CLICKABLE) && 
 	 mywidget->enable == WIDGET_ENABLE && mywidget->running) {
@@ -1100,6 +1113,8 @@ int xitk_click_notify_widget_list (xitk_widget_list_t *wl, int x, int y, int but
 	if(wl->widget_pressed->event(wl->widget_pressed, &event, &result))
 	  bRepaint |= result.value;
       }
+
+      wl->widget_pressed = NULL;
     }
   }
   
@@ -1197,7 +1212,7 @@ void xitk_set_focus_to_next_widget(xitk_widget_list_t *wl, int backward) {
     __focus_the_widget:
       
       /* Kill (hide) tips */
-      xitk_tips_tips_kill(wl->widget_focused);
+      xitk_tips_hide_tips();
 
       if ((wl->widget_focused->type & WIDGET_FOCUSABLE) &&
 	  (wl->widget_focused->enable == WIDGET_ENABLE)) {
@@ -1290,7 +1305,7 @@ void xitk_set_focus_to_widget(xitk_widget_t *w) {
     if(wl->widget_focused) {
       
       /* Kill (hide) tips */
-      xitk_tips_tips_kill(wl->widget_focused);
+      xitk_tips_hide_tips();
       
       if ((wl->widget_focused->type & WIDGET_FOCUSABLE) && 
 	  (wl->widget_focused->enable == WIDGET_ENABLE)) {
@@ -1507,7 +1522,7 @@ void xitk_disable_widget(xitk_widget_t *w) {
     
     if((w->wl->widget_under_mouse != NULL) && (w == w->wl->widget_under_mouse)) {
       /* Kill (hide) tips */
-      xitk_tips_tips_kill(w);
+      xitk_tips_hide_tips();
     }
     
     if((w->type & WIDGET_FOCUSABLE) && 
@@ -1519,6 +1534,8 @@ void xitk_disable_widget(xitk_widget_t *w) {
       
       event.type = WIDGET_EVENT_PAINT;
       (void) w->event(w, &event, NULL);
+
+      w->have_focus = FOCUS_LOST;
     }
   }
   
@@ -1542,7 +1559,7 @@ void xitk_free_widget(xitk_widget_t *w) {
   event.type = WIDGET_EVENT_DESTROY;
   (void) w->event(w, &event, NULL);
   
-  xitk_tips_tips_kill(w);
+  xitk_tips_hide_tips();
   XITK_FREE(w->tips_string);
   XITK_FREE(w);
   w = NULL;
@@ -1587,6 +1604,7 @@ void xitk_destroy_widgets(xitk_widget_list_t *wl) {
   }
 }
 
+#if 0
 /*
  * Return the struct of color names/values.
  */
@@ -1594,6 +1612,7 @@ xitk_color_names_t *gui_get_color_names(void) {
 
   return xitk_color_names;
 }
+#endif
 
 /*
  * Return a xitk_color_name_t type from a string color.
@@ -1733,7 +1752,7 @@ void xitk_show_widget(xitk_widget_t *w) {
 
   if(w->visible == 0) {
     widget_event_t  event;
-    
+
     w->visible = 1;
     
     event.type = WIDGET_EVENT_PAINT;
@@ -1791,7 +1810,7 @@ void xitk_hide_widget(xitk_widget_t *w) {
     
     w->visible = 0;
 
-    xitk_tips_tips_kill(w);
+    xitk_tips_hide_tips();
     
     event.type = WIDGET_EVENT_PAINT;
     (void) w->event(w, &event, NULL);
@@ -1901,13 +1920,13 @@ void xitk_set_widget_tips_default(xitk_widget_t *w, char *str) {
   }
 
   xitk_tips_set_tips(w, str);
-  xitk_tips_set_timeout(w, TIPS_TIMEOUT);
+  xitk_tips_set_timeout(w, xitk_get_tips_timeout());
 }
 
 /*
  *
  */
-void xitk_set_widget_tips_and_timeout(xitk_widget_t *w, char *str, unsigned int timeout) {
+void xitk_set_widget_tips_and_timeout(xitk_widget_t *w, char *str, unsigned long timeout) {
 
   if(!w || !str) {
     XITK_WARNING("widget is NULL\n");
@@ -1918,6 +1937,19 @@ void xitk_set_widget_tips_and_timeout(xitk_widget_t *w, char *str, unsigned int
   xitk_tips_set_timeout(w, timeout);
 }
 
+/*
+ *
+ */
+unsigned long xitk_get_widget_tips_timeout(xitk_widget_t *w) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return 0;
+  }
+  
+  return w->tips_timeout;
+}
+
 /*
  *
  */
@@ -1941,7 +1973,7 @@ void xitk_enable_widget_tips(xitk_widget_t *w) {
     return;
   }
   
-  xitk_tips_set_timeout(w, TIPS_TIMEOUT);
+  xitk_tips_set_timeout(w, xitk_get_tips_timeout());
 }
 
 /*
@@ -1992,12 +2024,14 @@ void xitk_enable_widgets_tips(xitk_widget_list_t *wl) {
  */
 void xitk_set_widgets_tips_timeout(xitk_widget_list_t *wl, unsigned long timeout) {
   xitk_widget_t *mywidget;
-  
+
   if(!wl) {
     XITK_WARNING("widget list was NULL.\n");
     return;
   }
 
+  xitk_set_tips_timeout(timeout);
+
   mywidget = (xitk_widget_t *) xitk_list_first_content (wl->l);
 
   while(mywidget) {
